"""File auto-generated by tglib TL generator. Do not edit manually."""
from ..tlobject import TLObject
from typing import Optional, List, Union
import os
import struct
from datetime import datetime


class BroadcastStats(TLObject):
    """TL type: stats.broadcastStats"""
    CONSTRUCTOR_ID = 0x396ca5fc
    SUBCLASS_OF_ID = 0x7ff25428

    def __init__(self, period: 'TypeStatsDateRangeDays', followers: 'TypeStatsAbsValueAndPrev', views_per_post: 'TypeStatsAbsValueAndPrev', shares_per_post: 'TypeStatsAbsValueAndPrev', reactions_per_post: 'TypeStatsAbsValueAndPrev', views_per_story: 'TypeStatsAbsValueAndPrev', shares_per_story: 'TypeStatsAbsValueAndPrev', reactions_per_story: 'TypeStatsAbsValueAndPrev', enabled_notifications: 'TypeStatsPercentValue', growth_graph: 'TypeStatsGraph', followers_graph: 'TypeStatsGraph', mute_graph: 'TypeStatsGraph', top_hours_graph: 'TypeStatsGraph', interactions_graph: 'TypeStatsGraph', iv_interactions_graph: 'TypeStatsGraph', views_by_source_graph: 'TypeStatsGraph', new_followers_by_source_graph: 'TypeStatsGraph', languages_graph: 'TypeStatsGraph', reactions_by_emotion_graph: 'TypeStatsGraph', story_interactions_graph: 'TypeStatsGraph', story_reactions_by_emotion_graph: 'TypeStatsGraph', recent_posts_interactions: List['TypePostInteractionCounters']):
        self.period = period
        self.followers = followers
        self.views_per_post = views_per_post
        self.shares_per_post = shares_per_post
        self.reactions_per_post = reactions_per_post
        self.views_per_story = views_per_story
        self.shares_per_story = shares_per_story
        self.reactions_per_story = reactions_per_story
        self.enabled_notifications = enabled_notifications
        self.growth_graph = growth_graph
        self.followers_graph = followers_graph
        self.mute_graph = mute_graph
        self.top_hours_graph = top_hours_graph
        self.interactions_graph = interactions_graph
        self.iv_interactions_graph = iv_interactions_graph
        self.views_by_source_graph = views_by_source_graph
        self.new_followers_by_source_graph = new_followers_by_source_graph
        self.languages_graph = languages_graph
        self.reactions_by_emotion_graph = reactions_by_emotion_graph
        self.story_interactions_graph = story_interactions_graph
        self.story_reactions_by_emotion_graph = story_reactions_by_emotion_graph
        self.recent_posts_interactions = recent_posts_interactions

    def to_dict(self):
        return {
            '_': 'BroadcastStats',
            'period': self.period,
            'followers': self.followers,
            'views_per_post': self.views_per_post,
            'shares_per_post': self.shares_per_post,
            'reactions_per_post': self.reactions_per_post,
            'views_per_story': self.views_per_story,
            'shares_per_story': self.shares_per_story,
            'reactions_per_story': self.reactions_per_story,
            'enabled_notifications': self.enabled_notifications,
            'growth_graph': self.growth_graph,
            'followers_graph': self.followers_graph,
            'mute_graph': self.mute_graph,
            'top_hours_graph': self.top_hours_graph,
            'interactions_graph': self.interactions_graph,
            'iv_interactions_graph': self.iv_interactions_graph,
            'views_by_source_graph': self.views_by_source_graph,
            'new_followers_by_source_graph': self.new_followers_by_source_graph,
            'languages_graph': self.languages_graph,
            'reactions_by_emotion_graph': self.reactions_by_emotion_graph,
            'story_interactions_graph': self.story_interactions_graph,
            'story_reactions_by_emotion_graph': self.story_reactions_by_emotion_graph,
            'recent_posts_interactions': self.recent_posts_interactions,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(bytes(self.period))
        buf.write(bytes(self.followers))
        buf.write(bytes(self.views_per_post))
        buf.write(bytes(self.shares_per_post))
        buf.write(bytes(self.reactions_per_post))
        buf.write(bytes(self.views_per_story))
        buf.write(bytes(self.shares_per_story))
        buf.write(bytes(self.reactions_per_story))
        buf.write(bytes(self.enabled_notifications))
        buf.write(bytes(self.growth_graph))
        buf.write(bytes(self.followers_graph))
        buf.write(bytes(self.mute_graph))
        buf.write(bytes(self.top_hours_graph))
        buf.write(bytes(self.interactions_graph))
        buf.write(bytes(self.iv_interactions_graph))
        buf.write(bytes(self.views_by_source_graph))
        buf.write(bytes(self.new_followers_by_source_graph))
        buf.write(bytes(self.languages_graph))
        buf.write(bytes(self.reactions_by_emotion_graph))
        buf.write(bytes(self.story_interactions_graph))
        buf.write(bytes(self.story_reactions_by_emotion_graph))
        buf.write(struct.pack('<i', 0x1cb5c415))  # vector id
        buf.write(struct.pack('<i', len(self.recent_posts_interactions)))
        for item in self.recent_posts_interactions:
            buf.write(bytes(self.item))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_period = reader.tgread_object()
        _args['period'] = _val_period
        _val_followers = reader.tgread_object()
        _args['followers'] = _val_followers
        _val_views_per_post = reader.tgread_object()
        _args['views_per_post'] = _val_views_per_post
        _val_shares_per_post = reader.tgread_object()
        _args['shares_per_post'] = _val_shares_per_post
        _val_reactions_per_post = reader.tgread_object()
        _args['reactions_per_post'] = _val_reactions_per_post
        _val_views_per_story = reader.tgread_object()
        _args['views_per_story'] = _val_views_per_story
        _val_shares_per_story = reader.tgread_object()
        _args['shares_per_story'] = _val_shares_per_story
        _val_reactions_per_story = reader.tgread_object()
        _args['reactions_per_story'] = _val_reactions_per_story
        _val_enabled_notifications = reader.tgread_object()
        _args['enabled_notifications'] = _val_enabled_notifications
        _val_growth_graph = reader.tgread_object()
        _args['growth_graph'] = _val_growth_graph
        _val_followers_graph = reader.tgread_object()
        _args['followers_graph'] = _val_followers_graph
        _val_mute_graph = reader.tgread_object()
        _args['mute_graph'] = _val_mute_graph
        _val_top_hours_graph = reader.tgread_object()
        _args['top_hours_graph'] = _val_top_hours_graph
        _val_interactions_graph = reader.tgread_object()
        _args['interactions_graph'] = _val_interactions_graph
        _val_iv_interactions_graph = reader.tgread_object()
        _args['iv_interactions_graph'] = _val_iv_interactions_graph
        _val_views_by_source_graph = reader.tgread_object()
        _args['views_by_source_graph'] = _val_views_by_source_graph
        _val_new_followers_by_source_graph = reader.tgread_object()
        _args['new_followers_by_source_graph'] = _val_new_followers_by_source_graph
        _val_languages_graph = reader.tgread_object()
        _args['languages_graph'] = _val_languages_graph
        _val_reactions_by_emotion_graph = reader.tgread_object()
        _args['reactions_by_emotion_graph'] = _val_reactions_by_emotion_graph
        _val_story_interactions_graph = reader.tgread_object()
        _args['story_interactions_graph'] = _val_story_interactions_graph
        _val_story_reactions_by_emotion_graph = reader.tgread_object()
        _args['story_reactions_by_emotion_graph'] = _val_story_reactions_by_emotion_graph
        reader.read_int(signed=False)  # skip vector id
        _count_recent_posts_interactions = reader.read_int()
        _list_recent_posts_interactions = []
        for _ in range(_count_recent_posts_interactions):
            _item_recent_posts_interactions = reader.tgread_object()
            _list_recent_posts_interactions.append(_item_recent_posts_interactions)
        _args['recent_posts_interactions'] = _list_recent_posts_interactions
        return cls(**_args)


class MegagroupStats(TLObject):
    """TL type: stats.megagroupStats"""
    CONSTRUCTOR_ID = 0xef7ff916
    SUBCLASS_OF_ID = 0x5b59be8d

    def __init__(self, period: 'TypeStatsDateRangeDays', members: 'TypeStatsAbsValueAndPrev', messages: 'TypeStatsAbsValueAndPrev', viewers: 'TypeStatsAbsValueAndPrev', posters: 'TypeStatsAbsValueAndPrev', growth_graph: 'TypeStatsGraph', members_graph: 'TypeStatsGraph', new_members_by_source_graph: 'TypeStatsGraph', languages_graph: 'TypeStatsGraph', messages_graph: 'TypeStatsGraph', actions_graph: 'TypeStatsGraph', top_hours_graph: 'TypeStatsGraph', weekdays_graph: 'TypeStatsGraph', top_posters: List['TypeStatsGroupTopPoster'], top_admins: List['TypeStatsGroupTopAdmin'], top_inviters: List['TypeStatsGroupTopInviter'], users: List['TypeUser']):
        self.period = period
        self.members = members
        self.messages = messages
        self.viewers = viewers
        self.posters = posters
        self.growth_graph = growth_graph
        self.members_graph = members_graph
        self.new_members_by_source_graph = new_members_by_source_graph
        self.languages_graph = languages_graph
        self.messages_graph = messages_graph
        self.actions_graph = actions_graph
        self.top_hours_graph = top_hours_graph
        self.weekdays_graph = weekdays_graph
        self.top_posters = top_posters
        self.top_admins = top_admins
        self.top_inviters = top_inviters
        self.users = users

    def to_dict(self):
        return {
            '_': 'MegagroupStats',
            'period': self.period,
            'members': self.members,
            'messages': self.messages,
            'viewers': self.viewers,
            'posters': self.posters,
            'growth_graph': self.growth_graph,
            'members_graph': self.members_graph,
            'new_members_by_source_graph': self.new_members_by_source_graph,
            'languages_graph': self.languages_graph,
            'messages_graph': self.messages_graph,
            'actions_graph': self.actions_graph,
            'top_hours_graph': self.top_hours_graph,
            'weekdays_graph': self.weekdays_graph,
            'top_posters': self.top_posters,
            'top_admins': self.top_admins,
            'top_inviters': self.top_inviters,
            'users': self.users,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(bytes(self.period))
        buf.write(bytes(self.members))
        buf.write(bytes(self.messages))
        buf.write(bytes(self.viewers))
        buf.write(bytes(self.posters))
        buf.write(bytes(self.growth_graph))
        buf.write(bytes(self.members_graph))
        buf.write(bytes(self.new_members_by_source_graph))
        buf.write(bytes(self.languages_graph))
        buf.write(bytes(self.messages_graph))
        buf.write(bytes(self.actions_graph))
        buf.write(bytes(self.top_hours_graph))
        buf.write(bytes(self.weekdays_graph))
        buf.write(struct.pack('<i', 0x1cb5c415))  # vector id
        buf.write(struct.pack('<i', len(self.top_posters)))
        for item in self.top_posters:
            buf.write(bytes(self.item))
        buf.write(struct.pack('<i', 0x1cb5c415))  # vector id
        buf.write(struct.pack('<i', len(self.top_admins)))
        for item in self.top_admins:
            buf.write(bytes(self.item))
        buf.write(struct.pack('<i', 0x1cb5c415))  # vector id
        buf.write(struct.pack('<i', len(self.top_inviters)))
        for item in self.top_inviters:
            buf.write(bytes(self.item))
        buf.write(struct.pack('<i', 0x1cb5c415))  # vector id
        buf.write(struct.pack('<i', len(self.users)))
        for item in self.users:
            buf.write(bytes(self.item))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_period = reader.tgread_object()
        _args['period'] = _val_period
        _val_members = reader.tgread_object()
        _args['members'] = _val_members
        _val_messages = reader.tgread_object()
        _args['messages'] = _val_messages
        _val_viewers = reader.tgread_object()
        _args['viewers'] = _val_viewers
        _val_posters = reader.tgread_object()
        _args['posters'] = _val_posters
        _val_growth_graph = reader.tgread_object()
        _args['growth_graph'] = _val_growth_graph
        _val_members_graph = reader.tgread_object()
        _args['members_graph'] = _val_members_graph
        _val_new_members_by_source_graph = reader.tgread_object()
        _args['new_members_by_source_graph'] = _val_new_members_by_source_graph
        _val_languages_graph = reader.tgread_object()
        _args['languages_graph'] = _val_languages_graph
        _val_messages_graph = reader.tgread_object()
        _args['messages_graph'] = _val_messages_graph
        _val_actions_graph = reader.tgread_object()
        _args['actions_graph'] = _val_actions_graph
        _val_top_hours_graph = reader.tgread_object()
        _args['top_hours_graph'] = _val_top_hours_graph
        _val_weekdays_graph = reader.tgread_object()
        _args['weekdays_graph'] = _val_weekdays_graph
        reader.read_int(signed=False)  # skip vector id
        _count_top_posters = reader.read_int()
        _list_top_posters = []
        for _ in range(_count_top_posters):
            _item_top_posters = reader.tgread_object()
            _list_top_posters.append(_item_top_posters)
        _args['top_posters'] = _list_top_posters
        reader.read_int(signed=False)  # skip vector id
        _count_top_admins = reader.read_int()
        _list_top_admins = []
        for _ in range(_count_top_admins):
            _item_top_admins = reader.tgread_object()
            _list_top_admins.append(_item_top_admins)
        _args['top_admins'] = _list_top_admins
        reader.read_int(signed=False)  # skip vector id
        _count_top_inviters = reader.read_int()
        _list_top_inviters = []
        for _ in range(_count_top_inviters):
            _item_top_inviters = reader.tgread_object()
            _list_top_inviters.append(_item_top_inviters)
        _args['top_inviters'] = _list_top_inviters
        reader.read_int(signed=False)  # skip vector id
        _count_users = reader.read_int()
        _list_users = []
        for _ in range(_count_users):
            _item_users = reader.tgread_object()
            _list_users.append(_item_users)
        _args['users'] = _list_users
        return cls(**_args)


class MessageStats(TLObject):
    """TL type: stats.messageStats"""
    CONSTRUCTOR_ID = 0x7fe91c14
    SUBCLASS_OF_ID = 0x9604a322

    def __init__(self, views_graph: 'TypeStatsGraph', reactions_by_emotion_graph: 'TypeStatsGraph'):
        self.views_graph = views_graph
        self.reactions_by_emotion_graph = reactions_by_emotion_graph

    def to_dict(self):
        return {
            '_': 'MessageStats',
            'views_graph': self.views_graph,
            'reactions_by_emotion_graph': self.reactions_by_emotion_graph,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(bytes(self.views_graph))
        buf.write(bytes(self.reactions_by_emotion_graph))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_views_graph = reader.tgread_object()
        _args['views_graph'] = _val_views_graph
        _val_reactions_by_emotion_graph = reader.tgread_object()
        _args['reactions_by_emotion_graph'] = _val_reactions_by_emotion_graph
        return cls(**_args)


class PublicForwards(TLObject):
    """TL type: stats.publicForwards"""
    CONSTRUCTOR_ID = 0x93037e20
    SUBCLASS_OF_ID = 0xa7283211

    def __init__(self, count: int, forwards: List['TypePublicForward'], chats: List['TypeChat'], users: List['TypeUser'], next_offset: Optional[str] = None):
        self.count = count
        self.forwards = forwards
        self.chats = chats
        self.users = users
        self.next_offset = next_offset

    def to_dict(self):
        return {
            '_': 'PublicForwards',
            'count': self.count,
            'forwards': self.forwards,
            'chats': self.chats,
            'users': self.users,
            'next_offset': self.next_offset,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        flags = 0
        if self.next_offset is not None:
            flags |= (1 << 0)
        buf.write(struct.pack('<I', flags))
        buf.write(struct.pack('<i', self.count))
        buf.write(struct.pack('<i', 0x1cb5c415))  # vector id
        buf.write(struct.pack('<i', len(self.forwards)))
        for item in self.forwards:
            buf.write(bytes(self.item))
        buf.write(struct.pack('<i', 0x1cb5c415))  # vector id
        buf.write(struct.pack('<i', len(self.chats)))
        for item in self.chats:
            buf.write(bytes(self.item))
        buf.write(struct.pack('<i', 0x1cb5c415))  # vector id
        buf.write(struct.pack('<i', len(self.users)))
        for item in self.users:
            buf.write(bytes(self.item))
        if self.next_offset is not None:
            buf.write(TLObject.serialize_bytes(self.next_offset))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        flags = reader.read_int(signed=False)
        _val_count = reader.read_int()
        _args['count'] = _val_count
        reader.read_int(signed=False)  # skip vector id
        _count_forwards = reader.read_int()
        _list_forwards = []
        for _ in range(_count_forwards):
            _item_forwards = reader.tgread_object()
            _list_forwards.append(_item_forwards)
        _args['forwards'] = _list_forwards
        if flags & (1 << 0):
            _val_next_offset = reader.tgread_string()
            _args['next_offset'] = _val_next_offset
        else:
            _args['next_offset'] = None
        reader.read_int(signed=False)  # skip vector id
        _count_chats = reader.read_int()
        _list_chats = []
        for _ in range(_count_chats):
            _item_chats = reader.tgread_object()
            _list_chats.append(_item_chats)
        _args['chats'] = _list_chats
        reader.read_int(signed=False)  # skip vector id
        _count_users = reader.read_int()
        _list_users = []
        for _ in range(_count_users):
            _item_users = reader.tgread_object()
            _list_users.append(_item_users)
        _args['users'] = _list_users
        return cls(**_args)


class StoryStats(TLObject):
    """TL type: stats.storyStats"""
    CONSTRUCTOR_ID = 0x50cd067c
    SUBCLASS_OF_ID = 0x8b4d43d4

    def __init__(self, views_graph: 'TypeStatsGraph', reactions_by_emotion_graph: 'TypeStatsGraph'):
        self.views_graph = views_graph
        self.reactions_by_emotion_graph = reactions_by_emotion_graph

    def to_dict(self):
        return {
            '_': 'StoryStats',
            'views_graph': self.views_graph,
            'reactions_by_emotion_graph': self.reactions_by_emotion_graph,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(bytes(self.views_graph))
        buf.write(bytes(self.reactions_by_emotion_graph))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_views_graph = reader.tgread_object()
        _args['views_graph'] = _val_views_graph
        _val_reactions_by_emotion_graph = reader.tgread_object()
        _args['reactions_by_emotion_graph'] = _val_reactions_by_emotion_graph
        return cls(**_args)




"""File auto-generated by tglib TL generator. Do not edit manually."""
from ..tlobject import TLObject
from ..tlobject import TLRequest
from typing import Optional, List, Union
import os
import struct
from datetime import datetime


class AddStickerToSetRequest(TLRequest):
    """TL type: stickers.addStickerToSet"""
    CONSTRUCTOR_ID = 0x8653febe
    SUBCLASS_OF_ID = 0x9b704a5a

    def __init__(self, stickerset: 'TypeInputStickerSet', sticker: 'TypeInputStickerSetItem'):
        self.stickerset = stickerset
        self.sticker = sticker

    def to_dict(self):
        return {
            '_': 'AddStickerToSetRequest',
            'stickerset': self.stickerset,
            'sticker': self.sticker,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(bytes(self.stickerset))
        buf.write(bytes(self.sticker))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_stickerset = reader.tgread_object()
        _args['stickerset'] = _val_stickerset
        _val_sticker = reader.tgread_object()
        _args['sticker'] = _val_sticker
        return cls(**_args)


class ChangeStickerRequest(TLRequest):
    """TL type: stickers.changeSticker"""
    CONSTRUCTOR_ID = 0xf5537ebc
    SUBCLASS_OF_ID = 0x9b704a5a

    def __init__(self, sticker: 'TypeInputDocument', emoji: Optional[str] = None, mask_coords: Optional['TypeMaskCoords'] = None, keywords: Optional[str] = None):
        self.sticker = sticker
        self.emoji = emoji
        self.mask_coords = mask_coords
        self.keywords = keywords

    def to_dict(self):
        return {
            '_': 'ChangeStickerRequest',
            'sticker': self.sticker,
            'emoji': self.emoji,
            'mask_coords': self.mask_coords,
            'keywords': self.keywords,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        flags = 0
        if self.emoji is not None:
            flags |= (1 << 0)
        if self.mask_coords is not None:
            flags |= (1 << 1)
        if self.keywords is not None:
            flags |= (1 << 2)
        buf.write(struct.pack('<I', flags))
        buf.write(bytes(self.sticker))
        if self.emoji is not None:
            buf.write(TLObject.serialize_bytes(self.emoji))
        if self.mask_coords is not None:
            buf.write(bytes(self.mask_coords))
        if self.keywords is not None:
            buf.write(TLObject.serialize_bytes(self.keywords))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        flags = reader.read_int(signed=False)
        _val_sticker = reader.tgread_object()
        _args['sticker'] = _val_sticker
        if flags & (1 << 0):
            _val_emoji = reader.tgread_string()
            _args['emoji'] = _val_emoji
        else:
            _args['emoji'] = None
        if flags & (1 << 1):
            _val_mask_coords = reader.tgread_object()
            _args['mask_coords'] = _val_mask_coords
        else:
            _args['mask_coords'] = None
        if flags & (1 << 2):
            _val_keywords = reader.tgread_string()
            _args['keywords'] = _val_keywords
        else:
            _args['keywords'] = None
        return cls(**_args)


class ChangeStickerPositionRequest(TLRequest):
    """TL type: stickers.changeStickerPosition"""
    CONSTRUCTOR_ID = 0xffb6d4ca
    SUBCLASS_OF_ID = 0x9b704a5a

    def __init__(self, sticker: 'TypeInputDocument', position: int):
        self.sticker = sticker
        self.position = position

    def to_dict(self):
        return {
            '_': 'ChangeStickerPositionRequest',
            'sticker': self.sticker,
            'position': self.position,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(bytes(self.sticker))
        buf.write(struct.pack('<i', self.position))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_sticker = reader.tgread_object()
        _args['sticker'] = _val_sticker
        _val_position = reader.read_int()
        _args['position'] = _val_position
        return cls(**_args)


class CheckShortNameRequest(TLRequest):
    """TL type: stickers.checkShortName"""
    CONSTRUCTOR_ID = 0x284b3639
    SUBCLASS_OF_ID = 0xf5b399ac

    def __init__(self, short_name: str):
        self.short_name = short_name

    def to_dict(self):
        return {
            '_': 'CheckShortNameRequest',
            'short_name': self.short_name,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(TLObject.serialize_bytes(self.short_name))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_short_name = reader.tgread_string()
        _args['short_name'] = _val_short_name
        return cls(**_args)


class CreateStickerSetRequest(TLRequest):
    """TL type: stickers.createStickerSet"""
    CONSTRUCTOR_ID = 0x9021ab67
    SUBCLASS_OF_ID = 0x9b704a5a

    def __init__(self, user_id: 'TypeInputUser', title: str, short_name: str, stickers: List['TypeInputStickerSetItem'], masks: Optional[bool] = None, emojis: Optional[bool] = None, text_color: Optional[bool] = None, thumb: Optional['TypeInputDocument'] = None, software: Optional[str] = None):
        self.user_id = user_id
        self.title = title
        self.short_name = short_name
        self.stickers = stickers
        self.masks = masks
        self.emojis = emojis
        self.text_color = text_color
        self.thumb = thumb
        self.software = software

    def to_dict(self):
        return {
            '_': 'CreateStickerSetRequest',
            'user_id': self.user_id,
            'title': self.title,
            'short_name': self.short_name,
            'stickers': self.stickers,
            'masks': self.masks,
            'emojis': self.emojis,
            'text_color': self.text_color,
            'thumb': self.thumb,
            'software': self.software,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        flags = 0
        if self.masks:
            flags |= (1 << 0)
        if self.emojis:
            flags |= (1 << 5)
        if self.text_color:
            flags |= (1 << 6)
        if self.thumb is not None:
            flags |= (1 << 2)
        if self.software is not None:
            flags |= (1 << 3)
        buf.write(struct.pack('<I', flags))
        buf.write(bytes(self.user_id))
        buf.write(TLObject.serialize_bytes(self.title))
        buf.write(TLObject.serialize_bytes(self.short_name))
        buf.write(struct.pack('<i', 0x1cb5c415))  # vector id
        buf.write(struct.pack('<i', len(self.stickers)))
        for item in self.stickers:
            buf.write(bytes(self.item))
        if self.thumb is not None:
            buf.write(bytes(self.thumb))
        if self.software is not None:
            buf.write(TLObject.serialize_bytes(self.software))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        flags = reader.read_int(signed=False)
        _args['masks'] = bool(flags & (1 << 0))
        _args['emojis'] = bool(flags & (1 << 5))
        _args['text_color'] = bool(flags & (1 << 6))
        _val_user_id = reader.tgread_object()
        _args['user_id'] = _val_user_id
        _val_title = reader.tgread_string()
        _args['title'] = _val_title
        _val_short_name = reader.tgread_string()
        _args['short_name'] = _val_short_name
        if flags & (1 << 2):
            _val_thumb = reader.tgread_object()
            _args['thumb'] = _val_thumb
        else:
            _args['thumb'] = None
        reader.read_int(signed=False)  # skip vector id
        _count_stickers = reader.read_int()
        _list_stickers = []
        for _ in range(_count_stickers):
            _item_stickers = reader.tgread_object()
            _list_stickers.append(_item_stickers)
        _args['stickers'] = _list_stickers
        if flags & (1 << 3):
            _val_software = reader.tgread_string()
            _args['software'] = _val_software
        else:
            _args['software'] = None
        return cls(**_args)


class DeleteStickerSetRequest(TLRequest):
    """TL type: stickers.deleteStickerSet"""
    CONSTRUCTOR_ID = 0x87704394
    SUBCLASS_OF_ID = 0xf5b399ac

    def __init__(self, stickerset: 'TypeInputStickerSet'):
        self.stickerset = stickerset

    def to_dict(self):
        return {
            '_': 'DeleteStickerSetRequest',
            'stickerset': self.stickerset,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(bytes(self.stickerset))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_stickerset = reader.tgread_object()
        _args['stickerset'] = _val_stickerset
        return cls(**_args)


class RemoveStickerFromSetRequest(TLRequest):
    """TL type: stickers.removeStickerFromSet"""
    CONSTRUCTOR_ID = 0xf7760f51
    SUBCLASS_OF_ID = 0x9b704a5a

    def __init__(self, sticker: 'TypeInputDocument'):
        self.sticker = sticker

    def to_dict(self):
        return {
            '_': 'RemoveStickerFromSetRequest',
            'sticker': self.sticker,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(bytes(self.sticker))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_sticker = reader.tgread_object()
        _args['sticker'] = _val_sticker
        return cls(**_args)


class RenameStickerSetRequest(TLRequest):
    """TL type: stickers.renameStickerSet"""
    CONSTRUCTOR_ID = 0x124b1c00
    SUBCLASS_OF_ID = 0x9b704a5a

    def __init__(self, stickerset: 'TypeInputStickerSet', title: str):
        self.stickerset = stickerset
        self.title = title

    def to_dict(self):
        return {
            '_': 'RenameStickerSetRequest',
            'stickerset': self.stickerset,
            'title': self.title,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(bytes(self.stickerset))
        buf.write(TLObject.serialize_bytes(self.title))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_stickerset = reader.tgread_object()
        _args['stickerset'] = _val_stickerset
        _val_title = reader.tgread_string()
        _args['title'] = _val_title
        return cls(**_args)


class ReplaceStickerRequest(TLRequest):
    """TL type: stickers.replaceSticker"""
    CONSTRUCTOR_ID = 0x4696459a
    SUBCLASS_OF_ID = 0x9b704a5a

    def __init__(self, sticker: 'TypeInputDocument', new_sticker: 'TypeInputStickerSetItem'):
        self.sticker = sticker
        self.new_sticker = new_sticker

    def to_dict(self):
        return {
            '_': 'ReplaceStickerRequest',
            'sticker': self.sticker,
            'new_sticker': self.new_sticker,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(bytes(self.sticker))
        buf.write(bytes(self.new_sticker))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_sticker = reader.tgread_object()
        _args['sticker'] = _val_sticker
        _val_new_sticker = reader.tgread_object()
        _args['new_sticker'] = _val_new_sticker
        return cls(**_args)


class SetStickerSetThumbRequest(TLRequest):
    """TL type: stickers.setStickerSetThumb"""
    CONSTRUCTOR_ID = 0xa76a5392
    SUBCLASS_OF_ID = 0x9b704a5a

    def __init__(self, stickerset: 'TypeInputStickerSet', thumb: Optional['TypeInputDocument'] = None, thumb_document_id: Optional[int] = None):
        self.stickerset = stickerset
        self.thumb = thumb
        self.thumb_document_id = thumb_document_id

    def to_dict(self):
        return {
            '_': 'SetStickerSetThumbRequest',
            'stickerset': self.stickerset,
            'thumb': self.thumb,
            'thumb_document_id': self.thumb_document_id,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        flags = 0
        if self.thumb is not None:
            flags |= (1 << 0)
        if self.thumb_document_id is not None:
            flags |= (1 << 1)
        buf.write(struct.pack('<I', flags))
        buf.write(bytes(self.stickerset))
        if self.thumb is not None:
            buf.write(bytes(self.thumb))
        if self.thumb_document_id is not None:
            buf.write(struct.pack('<q', self.thumb_document_id))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        flags = reader.read_int(signed=False)
        _val_stickerset = reader.tgread_object()
        _args['stickerset'] = _val_stickerset
        if flags & (1 << 0):
            _val_thumb = reader.tgread_object()
            _args['thumb'] = _val_thumb
        else:
            _args['thumb'] = None
        if flags & (1 << 1):
            _val_thumb_document_id = reader.read_long()
            _args['thumb_document_id'] = _val_thumb_document_id
        else:
            _args['thumb_document_id'] = None
        return cls(**_args)


class SuggestShortNameRequest(TLRequest):
    """TL type: stickers.suggestShortName"""
    CONSTRUCTOR_ID = 0x4dafc503
    SUBCLASS_OF_ID = 0xc44a4b21

    def __init__(self, title: str):
        self.title = title

    def to_dict(self):
        return {
            '_': 'SuggestShortNameRequest',
            'title': self.title,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(TLObject.serialize_bytes(self.title))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_title = reader.tgread_string()
        _args['title'] = _val_title
        return cls(**_args)




"""File auto-generated by tglib TL generator. Do not edit manually."""
from ..tlobject import TLObject
from typing import Optional, List, Union
import os
import struct
from datetime import datetime


class Blocked(TLObject):
    """TL type: contacts.blocked"""
    CONSTRUCTOR_ID = 0x0ade1591
    SUBCLASS_OF_ID = 0xffba4f4f

    def __init__(self, blocked: List['TypePeerBlocked'], chats: List['TypeChat'], users: List['TypeUser']):
        self.blocked = blocked
        self.chats = chats
        self.users = users

    def to_dict(self):
        return {
            '_': 'Blocked',
            'blocked': self.blocked,
            'chats': self.chats,
            'users': self.users,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(struct.pack('<i', 0x1cb5c415))  # vector id
        buf.write(struct.pack('<i', len(self.blocked)))
        for item in self.blocked:
            buf.write(bytes(self.item))
        buf.write(struct.pack('<i', 0x1cb5c415))  # vector id
        buf.write(struct.pack('<i', len(self.chats)))
        for item in self.chats:
            buf.write(bytes(self.item))
        buf.write(struct.pack('<i', 0x1cb5c415))  # vector id
        buf.write(struct.pack('<i', len(self.users)))
        for item in self.users:
            buf.write(bytes(self.item))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        reader.read_int(signed=False)  # skip vector id
        _count_blocked = reader.read_int()
        _list_blocked = []
        for _ in range(_count_blocked):
            _item_blocked = reader.tgread_object()
            _list_blocked.append(_item_blocked)
        _args['blocked'] = _list_blocked
        reader.read_int(signed=False)  # skip vector id
        _count_chats = reader.read_int()
        _list_chats = []
        for _ in range(_count_chats):
            _item_chats = reader.tgread_object()
            _list_chats.append(_item_chats)
        _args['chats'] = _list_chats
        reader.read_int(signed=False)  # skip vector id
        _count_users = reader.read_int()
        _list_users = []
        for _ in range(_count_users):
            _item_users = reader.tgread_object()
            _list_users.append(_item_users)
        _args['users'] = _list_users
        return cls(**_args)


class BlockedSlice(TLObject):
    """TL type: contacts.blockedSlice"""
    CONSTRUCTOR_ID = 0xe1664194
    SUBCLASS_OF_ID = 0xffba4f4f

    def __init__(self, count: int, blocked: List['TypePeerBlocked'], chats: List['TypeChat'], users: List['TypeUser']):
        self.count = count
        self.blocked = blocked
        self.chats = chats
        self.users = users

    def to_dict(self):
        return {
            '_': 'BlockedSlice',
            'count': self.count,
            'blocked': self.blocked,
            'chats': self.chats,
            'users': self.users,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(struct.pack('<i', self.count))
        buf.write(struct.pack('<i', 0x1cb5c415))  # vector id
        buf.write(struct.pack('<i', len(self.blocked)))
        for item in self.blocked:
            buf.write(bytes(self.item))
        buf.write(struct.pack('<i', 0x1cb5c415))  # vector id
        buf.write(struct.pack('<i', len(self.chats)))
        for item in self.chats:
            buf.write(bytes(self.item))
        buf.write(struct.pack('<i', 0x1cb5c415))  # vector id
        buf.write(struct.pack('<i', len(self.users)))
        for item in self.users:
            buf.write(bytes(self.item))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_count = reader.read_int()
        _args['count'] = _val_count
        reader.read_int(signed=False)  # skip vector id
        _count_blocked = reader.read_int()
        _list_blocked = []
        for _ in range(_count_blocked):
            _item_blocked = reader.tgread_object()
            _list_blocked.append(_item_blocked)
        _args['blocked'] = _list_blocked
        reader.read_int(signed=False)  # skip vector id
        _count_chats = reader.read_int()
        _list_chats = []
        for _ in range(_count_chats):
            _item_chats = reader.tgread_object()
            _list_chats.append(_item_chats)
        _args['chats'] = _list_chats
        reader.read_int(signed=False)  # skip vector id
        _count_users = reader.read_int()
        _list_users = []
        for _ in range(_count_users):
            _item_users = reader.tgread_object()
            _list_users.append(_item_users)
        _args['users'] = _list_users
        return cls(**_args)


class ContactBirthdays(TLObject):
    """TL type: contacts.contactBirthdays"""
    CONSTRUCTOR_ID = 0x114ff30d
    SUBCLASS_OF_ID = 0x0e7aabff

    def __init__(self, contacts: List['TypeContactBirthday'], users: List['TypeUser']):
        self.contacts = contacts
        self.users = users

    def to_dict(self):
        return {
            '_': 'ContactBirthdays',
            'contacts': self.contacts,
            'users': self.users,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(struct.pack('<i', 0x1cb5c415))  # vector id
        buf.write(struct.pack('<i', len(self.contacts)))
        for item in self.contacts:
            buf.write(bytes(self.item))
        buf.write(struct.pack('<i', 0x1cb5c415))  # vector id
        buf.write(struct.pack('<i', len(self.users)))
        for item in self.users:
            buf.write(bytes(self.item))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        reader.read_int(signed=False)  # skip vector id
        _count_contacts = reader.read_int()
        _list_contacts = []
        for _ in range(_count_contacts):
            _item_contacts = reader.tgread_object()
            _list_contacts.append(_item_contacts)
        _args['contacts'] = _list_contacts
        reader.read_int(signed=False)  # skip vector id
        _count_users = reader.read_int()
        _list_users = []
        for _ in range(_count_users):
            _item_users = reader.tgread_object()
            _list_users.append(_item_users)
        _args['users'] = _list_users
        return cls(**_args)


class Contacts(TLObject):
    """TL type: contacts.contacts"""
    CONSTRUCTOR_ID = 0xeae87e42
    SUBCLASS_OF_ID = 0x38be25f6

    def __init__(self, contacts: List['TypeContact'], saved_count: int, users: List['TypeUser']):
        self.contacts = contacts
        self.saved_count = saved_count
        self.users = users

    def to_dict(self):
        return {
            '_': 'Contacts',
            'contacts': self.contacts,
            'saved_count': self.saved_count,
            'users': self.users,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(struct.pack('<i', 0x1cb5c415))  # vector id
        buf.write(struct.pack('<i', len(self.contacts)))
        for item in self.contacts:
            buf.write(bytes(self.item))
        buf.write(struct.pack('<i', self.saved_count))
        buf.write(struct.pack('<i', 0x1cb5c415))  # vector id
        buf.write(struct.pack('<i', len(self.users)))
        for item in self.users:
            buf.write(bytes(self.item))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        reader.read_int(signed=False)  # skip vector id
        _count_contacts = reader.read_int()
        _list_contacts = []
        for _ in range(_count_contacts):
            _item_contacts = reader.tgread_object()
            _list_contacts.append(_item_contacts)
        _args['contacts'] = _list_contacts
        _val_saved_count = reader.read_int()
        _args['saved_count'] = _val_saved_count
        reader.read_int(signed=False)  # skip vector id
        _count_users = reader.read_int()
        _list_users = []
        for _ in range(_count_users):
            _item_users = reader.tgread_object()
            _list_users.append(_item_users)
        _args['users'] = _list_users
        return cls(**_args)


class ContactsNotModified(TLObject):
    """TL type: contacts.contactsNotModified"""
    CONSTRUCTOR_ID = 0xb74ba9d2
    SUBCLASS_OF_ID = 0x38be25f6

    def __init__(self):
        pass

    def to_dict(self):
        return {
            '_': 'ContactsNotModified',
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        return cls()


class Found(TLObject):
    """TL type: contacts.found"""
    CONSTRUCTOR_ID = 0xb3134d9d
    SUBCLASS_OF_ID = 0x4386a2e3

    def __init__(self, my_results: List['TypePeer'], results: List['TypePeer'], chats: List['TypeChat'], users: List['TypeUser']):
        self.my_results = my_results
        self.results = results
        self.chats = chats
        self.users = users

    def to_dict(self):
        return {
            '_': 'Found',
            'my_results': self.my_results,
            'results': self.results,
            'chats': self.chats,
            'users': self.users,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(struct.pack('<i', 0x1cb5c415))  # vector id
        buf.write(struct.pack('<i', len(self.my_results)))
        for item in self.my_results:
            buf.write(bytes(self.item))
        buf.write(struct.pack('<i', 0x1cb5c415))  # vector id
        buf.write(struct.pack('<i', len(self.results)))
        for item in self.results:
            buf.write(bytes(self.item))
        buf.write(struct.pack('<i', 0x1cb5c415))  # vector id
        buf.write(struct.pack('<i', len(self.chats)))
        for item in self.chats:
            buf.write(bytes(self.item))
        buf.write(struct.pack('<i', 0x1cb5c415))  # vector id
        buf.write(struct.pack('<i', len(self.users)))
        for item in self.users:
            buf.write(bytes(self.item))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        reader.read_int(signed=False)  # skip vector id
        _count_my_results = reader.read_int()
        _list_my_results = []
        for _ in range(_count_my_results):
            _item_my_results = reader.tgread_object()
            _list_my_results.append(_item_my_results)
        _args['my_results'] = _list_my_results
        reader.read_int(signed=False)  # skip vector id
        _count_results = reader.read_int()
        _list_results = []
        for _ in range(_count_results):
            _item_results = reader.tgread_object()
            _list_results.append(_item_results)
        _args['results'] = _list_results
        reader.read_int(signed=False)  # skip vector id
        _count_chats = reader.read_int()
        _list_chats = []
        for _ in range(_count_chats):
            _item_chats = reader.tgread_object()
            _list_chats.append(_item_chats)
        _args['chats'] = _list_chats
        reader.read_int(signed=False)  # skip vector id
        _count_users = reader.read_int()
        _list_users = []
        for _ in range(_count_users):
            _item_users = reader.tgread_object()
            _list_users.append(_item_users)
        _args['users'] = _list_users
        return cls(**_args)


class ImportedContacts(TLObject):
    """TL type: contacts.importedContacts"""
    CONSTRUCTOR_ID = 0x77d01c3b
    SUBCLASS_OF_ID = 0x8172ad93

    def __init__(self, imported: List['TypeImportedContact'], popular_invites: List['TypePopularContact'], retry_contacts: List[int], users: List['TypeUser']):
        self.imported = imported
        self.popular_invites = popular_invites
        self.retry_contacts = retry_contacts
        self.users = users

    def to_dict(self):
        return {
            '_': 'ImportedContacts',
            'imported': self.imported,
            'popular_invites': self.popular_invites,
            'retry_contacts': self.retry_contacts,
            'users': self.users,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(struct.pack('<i', 0x1cb5c415))  # vector id
        buf.write(struct.pack('<i', len(self.imported)))
        for item in self.imported:
            buf.write(bytes(self.item))
        buf.write(struct.pack('<i', 0x1cb5c415))  # vector id
        buf.write(struct.pack('<i', len(self.popular_invites)))
        for item in self.popular_invites:
            buf.write(bytes(self.item))
        buf.write(struct.pack('<i', 0x1cb5c415))  # vector id
        buf.write(struct.pack('<i', len(self.retry_contacts)))
        for item in self.retry_contacts:
            buf.write(struct.pack('<q', self.item))
        buf.write(struct.pack('<i', 0x1cb5c415))  # vector id
        buf.write(struct.pack('<i', len(self.users)))
        for item in self.users:
            buf.write(bytes(self.item))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        reader.read_int(signed=False)  # skip vector id
        _count_imported = reader.read_int()
        _list_imported = []
        for _ in range(_count_imported):
            _item_imported = reader.tgread_object()
            _list_imported.append(_item_imported)
        _args['imported'] = _list_imported
        reader.read_int(signed=False)  # skip vector id
        _count_popular_invites = reader.read_int()
        _list_popular_invites = []
        for _ in range(_count_popular_invites):
            _item_popular_invites = reader.tgread_object()
            _list_popular_invites.append(_item_popular_invites)
        _args['popular_invites'] = _list_popular_invites
        reader.read_int(signed=False)  # skip vector id
        _count_retry_contacts = reader.read_int()
        _list_retry_contacts = []
        for _ in range(_count_retry_contacts):
            _item_retry_contacts = reader.read_long()
            _list_retry_contacts.append(_item_retry_contacts)
        _args['retry_contacts'] = _list_retry_contacts
        reader.read_int(signed=False)  # skip vector id
        _count_users = reader.read_int()
        _list_users = []
        for _ in range(_count_users):
            _item_users = reader.tgread_object()
            _list_users.append(_item_users)
        _args['users'] = _list_users
        return cls(**_args)


class ResolvedPeer(TLObject):
    """TL type: contacts.resolvedPeer"""
    CONSTRUCTOR_ID = 0x7f077ad9
    SUBCLASS_OF_ID = 0xf065b3a8

    def __init__(self, peer: 'TypePeer', chats: List['TypeChat'], users: List['TypeUser']):
        self.peer = peer
        self.chats = chats
        self.users = users

    def to_dict(self):
        return {
            '_': 'ResolvedPeer',
            'peer': self.peer,
            'chats': self.chats,
            'users': self.users,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(bytes(self.peer))
        buf.write(struct.pack('<i', 0x1cb5c415))  # vector id
        buf.write(struct.pack('<i', len(self.chats)))
        for item in self.chats:
            buf.write(bytes(self.item))
        buf.write(struct.pack('<i', 0x1cb5c415))  # vector id
        buf.write(struct.pack('<i', len(self.users)))
        for item in self.users:
            buf.write(bytes(self.item))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_peer = reader.tgread_object()
        _args['peer'] = _val_peer
        reader.read_int(signed=False)  # skip vector id
        _count_chats = reader.read_int()
        _list_chats = []
        for _ in range(_count_chats):
            _item_chats = reader.tgread_object()
            _list_chats.append(_item_chats)
        _args['chats'] = _list_chats
        reader.read_int(signed=False)  # skip vector id
        _count_users = reader.read_int()
        _list_users = []
        for _ in range(_count_users):
            _item_users = reader.tgread_object()
            _list_users.append(_item_users)
        _args['users'] = _list_users
        return cls(**_args)


class SponsoredPeers(TLObject):
    """TL type: contacts.sponsoredPeers"""
    CONSTRUCTOR_ID = 0xeb032884
    SUBCLASS_OF_ID = 0xb45d5ccc

    def __init__(self, peers: List['TypeSponsoredPeer'], chats: List['TypeChat'], users: List['TypeUser']):
        self.peers = peers
        self.chats = chats
        self.users = users

    def to_dict(self):
        return {
            '_': 'SponsoredPeers',
            'peers': self.peers,
            'chats': self.chats,
            'users': self.users,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(struct.pack('<i', 0x1cb5c415))  # vector id
        buf.write(struct.pack('<i', len(self.peers)))
        for item in self.peers:
            buf.write(bytes(self.item))
        buf.write(struct.pack('<i', 0x1cb5c415))  # vector id
        buf.write(struct.pack('<i', len(self.chats)))
        for item in self.chats:
            buf.write(bytes(self.item))
        buf.write(struct.pack('<i', 0x1cb5c415))  # vector id
        buf.write(struct.pack('<i', len(self.users)))
        for item in self.users:
            buf.write(bytes(self.item))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        reader.read_int(signed=False)  # skip vector id
        _count_peers = reader.read_int()
        _list_peers = []
        for _ in range(_count_peers):
            _item_peers = reader.tgread_object()
            _list_peers.append(_item_peers)
        _args['peers'] = _list_peers
        reader.read_int(signed=False)  # skip vector id
        _count_chats = reader.read_int()
        _list_chats = []
        for _ in range(_count_chats):
            _item_chats = reader.tgread_object()
            _list_chats.append(_item_chats)
        _args['chats'] = _list_chats
        reader.read_int(signed=False)  # skip vector id
        _count_users = reader.read_int()
        _list_users = []
        for _ in range(_count_users):
            _item_users = reader.tgread_object()
            _list_users.append(_item_users)
        _args['users'] = _list_users
        return cls(**_args)


class SponsoredPeersEmpty(TLObject):
    """TL type: contacts.sponsoredPeersEmpty"""
    CONSTRUCTOR_ID = 0xea32b4b1
    SUBCLASS_OF_ID = 0xb45d5ccc

    def __init__(self):
        pass

    def to_dict(self):
        return {
            '_': 'SponsoredPeersEmpty',
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        return cls()


class TopPeers(TLObject):
    """TL type: contacts.topPeers"""
    CONSTRUCTOR_ID = 0x70b772a8
    SUBCLASS_OF_ID = 0x9ee8bb88

    def __init__(self, categories: List['TypeTopPeerCategoryPeers'], chats: List['TypeChat'], users: List['TypeUser']):
        self.categories = categories
        self.chats = chats
        self.users = users

    def to_dict(self):
        return {
            '_': 'TopPeers',
            'categories': self.categories,
            'chats': self.chats,
            'users': self.users,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(struct.pack('<i', 0x1cb5c415))  # vector id
        buf.write(struct.pack('<i', len(self.categories)))
        for item in self.categories:
            buf.write(bytes(self.item))
        buf.write(struct.pack('<i', 0x1cb5c415))  # vector id
        buf.write(struct.pack('<i', len(self.chats)))
        for item in self.chats:
            buf.write(bytes(self.item))
        buf.write(struct.pack('<i', 0x1cb5c415))  # vector id
        buf.write(struct.pack('<i', len(self.users)))
        for item in self.users:
            buf.write(bytes(self.item))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        reader.read_int(signed=False)  # skip vector id
        _count_categories = reader.read_int()
        _list_categories = []
        for _ in range(_count_categories):
            _item_categories = reader.tgread_object()
            _list_categories.append(_item_categories)
        _args['categories'] = _list_categories
        reader.read_int(signed=False)  # skip vector id
        _count_chats = reader.read_int()
        _list_chats = []
        for _ in range(_count_chats):
            _item_chats = reader.tgread_object()
            _list_chats.append(_item_chats)
        _args['chats'] = _list_chats
        reader.read_int(signed=False)  # skip vector id
        _count_users = reader.read_int()
        _list_users = []
        for _ in range(_count_users):
            _item_users = reader.tgread_object()
            _list_users.append(_item_users)
        _args['users'] = _list_users
        return cls(**_args)


class TopPeersDisabled(TLObject):
    """TL type: contacts.topPeersDisabled"""
    CONSTRUCTOR_ID = 0xb52c939d
    SUBCLASS_OF_ID = 0x9ee8bb88

    def __init__(self):
        pass

    def to_dict(self):
        return {
            '_': 'TopPeersDisabled',
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        return cls()


class TopPeersNotModified(TLObject):
    """TL type: contacts.topPeersNotModified"""
    CONSTRUCTOR_ID = 0xde266ef5
    SUBCLASS_OF_ID = 0x9ee8bb88

    def __init__(self):
        pass

    def to_dict(self):
        return {
            '_': 'TopPeersNotModified',
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        return cls()




"""File auto-generated by tglib TL generator. Do not edit manually."""
from ..tlobject import TLObject
from typing import Optional, List, Union
import os
import struct
from datetime import datetime


class AuthorizationForm(TLObject):
    """TL type: account.authorizationForm"""
    CONSTRUCTOR_ID = 0xad2e1cd8
    SUBCLASS_OF_ID = 0x78049a94

    def __init__(self, required_types: List['TypeSecureRequiredType'], values: List['TypeSecureValue'], errors: List['TypeSecureValueError'], users: List['TypeUser'], privacy_policy_url: Optional[str] = None):
        self.required_types = required_types
        self.values = values
        self.errors = errors
        self.users = users
        self.privacy_policy_url = privacy_policy_url

    def to_dict(self):
        return {
            '_': 'AuthorizationForm',
            'required_types': self.required_types,
            'values': self.values,
            'errors': self.errors,
            'users': self.users,
            'privacy_policy_url': self.privacy_policy_url,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        flags = 0
        if self.privacy_policy_url is not None:
            flags |= (1 << 0)
        buf.write(struct.pack('<I', flags))
        buf.write(struct.pack('<i', 0x1cb5c415))  # vector id
        buf.write(struct.pack('<i', len(self.required_types)))
        for item in self.required_types:
            buf.write(bytes(self.item))
        buf.write(struct.pack('<i', 0x1cb5c415))  # vector id
        buf.write(struct.pack('<i', len(self.values)))
        for item in self.values:
            buf.write(bytes(self.item))
        buf.write(struct.pack('<i', 0x1cb5c415))  # vector id
        buf.write(struct.pack('<i', len(self.errors)))
        for item in self.errors:
            buf.write(bytes(self.item))
        buf.write(struct.pack('<i', 0x1cb5c415))  # vector id
        buf.write(struct.pack('<i', len(self.users)))
        for item in self.users:
            buf.write(bytes(self.item))
        if self.privacy_policy_url is not None:
            buf.write(TLObject.serialize_bytes(self.privacy_policy_url))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        flags = reader.read_int(signed=False)
        reader.read_int(signed=False)  # skip vector id
        _count_required_types = reader.read_int()
        _list_required_types = []
        for _ in range(_count_required_types):
            _item_required_types = reader.tgread_object()
            _list_required_types.append(_item_required_types)
        _args['required_types'] = _list_required_types
        reader.read_int(signed=False)  # skip vector id
        _count_values = reader.read_int()
        _list_values = []
        for _ in range(_count_values):
            _item_values = reader.tgread_object()
            _list_values.append(_item_values)
        _args['values'] = _list_values
        reader.read_int(signed=False)  # skip vector id
        _count_errors = reader.read_int()
        _list_errors = []
        for _ in range(_count_errors):
            _item_errors = reader.tgread_object()
            _list_errors.append(_item_errors)
        _args['errors'] = _list_errors
        reader.read_int(signed=False)  # skip vector id
        _count_users = reader.read_int()
        _list_users = []
        for _ in range(_count_users):
            _item_users = reader.tgread_object()
            _list_users.append(_item_users)
        _args['users'] = _list_users
        if flags & (1 << 0):
            _val_privacy_policy_url = reader.tgread_string()
            _args['privacy_policy_url'] = _val_privacy_policy_url
        else:
            _args['privacy_policy_url'] = None
        return cls(**_args)


class Authorizations(TLObject):
    """TL type: account.authorizations"""
    CONSTRUCTOR_ID = 0x4bff8ea0
    SUBCLASS_OF_ID = 0x0bf5e0ff

    def __init__(self, authorization_ttl_days: int, authorizations: List['TypeAuthorization']):
        self.authorization_ttl_days = authorization_ttl_days
        self.authorizations = authorizations

    def to_dict(self):
        return {
            '_': 'Authorizations',
            'authorization_ttl_days': self.authorization_ttl_days,
            'authorizations': self.authorizations,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(struct.pack('<i', self.authorization_ttl_days))
        buf.write(struct.pack('<i', 0x1cb5c415))  # vector id
        buf.write(struct.pack('<i', len(self.authorizations)))
        for item in self.authorizations:
            buf.write(bytes(self.item))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_authorization_ttl_days = reader.read_int()
        _args['authorization_ttl_days'] = _val_authorization_ttl_days
        reader.read_int(signed=False)  # skip vector id
        _count_authorizations = reader.read_int()
        _list_authorizations = []
        for _ in range(_count_authorizations):
            _item_authorizations = reader.tgread_object()
            _list_authorizations.append(_item_authorizations)
        _args['authorizations'] = _list_authorizations
        return cls(**_args)


class AutoDownloadSettings(TLObject):
    """TL type: account.autoDownloadSettings"""
    CONSTRUCTOR_ID = 0x63cacf26
    SUBCLASS_OF_ID = 0x2fb85921

    def __init__(self, low: 'TypeAutoDownloadSettings', medium: 'TypeAutoDownloadSettings', high: 'TypeAutoDownloadSettings'):
        self.low = low
        self.medium = medium
        self.high = high

    def to_dict(self):
        return {
            '_': 'AutoDownloadSettings',
            'low': self.low,
            'medium': self.medium,
            'high': self.high,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(bytes(self.low))
        buf.write(bytes(self.medium))
        buf.write(bytes(self.high))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_low = reader.tgread_object()
        _args['low'] = _val_low
        _val_medium = reader.tgread_object()
        _args['medium'] = _val_medium
        _val_high = reader.tgread_object()
        _args['high'] = _val_high
        return cls(**_args)


class AutoSaveSettings(TLObject):
    """TL type: account.autoSaveSettings"""
    CONSTRUCTOR_ID = 0x4c3e069d
    SUBCLASS_OF_ID = 0x48cf2f02

    def __init__(self, users_settings: 'TypeAutoSaveSettings', chats_settings: 'TypeAutoSaveSettings', broadcasts_settings: 'TypeAutoSaveSettings', exceptions: List['TypeAutoSaveException'], chats: List['TypeChat'], users: List['TypeUser']):
        self.users_settings = users_settings
        self.chats_settings = chats_settings
        self.broadcasts_settings = broadcasts_settings
        self.exceptions = exceptions
        self.chats = chats
        self.users = users

    def to_dict(self):
        return {
            '_': 'AutoSaveSettings',
            'users_settings': self.users_settings,
            'chats_settings': self.chats_settings,
            'broadcasts_settings': self.broadcasts_settings,
            'exceptions': self.exceptions,
            'chats': self.chats,
            'users': self.users,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(bytes(self.users_settings))
        buf.write(bytes(self.chats_settings))
        buf.write(bytes(self.broadcasts_settings))
        buf.write(struct.pack('<i', 0x1cb5c415))  # vector id
        buf.write(struct.pack('<i', len(self.exceptions)))
        for item in self.exceptions:
            buf.write(bytes(self.item))
        buf.write(struct.pack('<i', 0x1cb5c415))  # vector id
        buf.write(struct.pack('<i', len(self.chats)))
        for item in self.chats:
            buf.write(bytes(self.item))
        buf.write(struct.pack('<i', 0x1cb5c415))  # vector id
        buf.write(struct.pack('<i', len(self.users)))
        for item in self.users:
            buf.write(bytes(self.item))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_users_settings = reader.tgread_object()
        _args['users_settings'] = _val_users_settings
        _val_chats_settings = reader.tgread_object()
        _args['chats_settings'] = _val_chats_settings
        _val_broadcasts_settings = reader.tgread_object()
        _args['broadcasts_settings'] = _val_broadcasts_settings
        reader.read_int(signed=False)  # skip vector id
        _count_exceptions = reader.read_int()
        _list_exceptions = []
        for _ in range(_count_exceptions):
            _item_exceptions = reader.tgread_object()
            _list_exceptions.append(_item_exceptions)
        _args['exceptions'] = _list_exceptions
        reader.read_int(signed=False)  # skip vector id
        _count_chats = reader.read_int()
        _list_chats = []
        for _ in range(_count_chats):
            _item_chats = reader.tgread_object()
            _list_chats.append(_item_chats)
        _args['chats'] = _list_chats
        reader.read_int(signed=False)  # skip vector id
        _count_users = reader.read_int()
        _list_users = []
        for _ in range(_count_users):
            _item_users = reader.tgread_object()
            _list_users.append(_item_users)
        _args['users'] = _list_users
        return cls(**_args)


class BusinessChatLinks(TLObject):
    """TL type: account.businessChatLinks"""
    CONSTRUCTOR_ID = 0xec43a2d1
    SUBCLASS_OF_ID = 0xc6ba4a31

    def __init__(self, links: List['TypeBusinessChatLink'], chats: List['TypeChat'], users: List['TypeUser']):
        self.links = links
        self.chats = chats
        self.users = users

    def to_dict(self):
        return {
            '_': 'BusinessChatLinks',
            'links': self.links,
            'chats': self.chats,
            'users': self.users,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(struct.pack('<i', 0x1cb5c415))  # vector id
        buf.write(struct.pack('<i', len(self.links)))
        for item in self.links:
            buf.write(bytes(self.item))
        buf.write(struct.pack('<i', 0x1cb5c415))  # vector id
        buf.write(struct.pack('<i', len(self.chats)))
        for item in self.chats:
            buf.write(bytes(self.item))
        buf.write(struct.pack('<i', 0x1cb5c415))  # vector id
        buf.write(struct.pack('<i', len(self.users)))
        for item in self.users:
            buf.write(bytes(self.item))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        reader.read_int(signed=False)  # skip vector id
        _count_links = reader.read_int()
        _list_links = []
        for _ in range(_count_links):
            _item_links = reader.tgread_object()
            _list_links.append(_item_links)
        _args['links'] = _list_links
        reader.read_int(signed=False)  # skip vector id
        _count_chats = reader.read_int()
        _list_chats = []
        for _ in range(_count_chats):
            _item_chats = reader.tgread_object()
            _list_chats.append(_item_chats)
        _args['chats'] = _list_chats
        reader.read_int(signed=False)  # skip vector id
        _count_users = reader.read_int()
        _list_users = []
        for _ in range(_count_users):
            _item_users = reader.tgread_object()
            _list_users.append(_item_users)
        _args['users'] = _list_users
        return cls(**_args)


class ChatThemes(TLObject):
    """TL type: account.chatThemes"""
    CONSTRUCTOR_ID = 0xbe098173
    SUBCLASS_OF_ID = 0x15c14aa8

    def __init__(self, hash: int, themes: List['TypeChatTheme'], chats: List['TypeChat'], users: List['TypeUser'], next_offset: Optional[str] = None):
        self.hash = hash
        self.themes = themes
        self.chats = chats
        self.users = users
        self.next_offset = next_offset

    def to_dict(self):
        return {
            '_': 'ChatThemes',
            'hash': self.hash,
            'themes': self.themes,
            'chats': self.chats,
            'users': self.users,
            'next_offset': self.next_offset,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        flags = 0
        if self.next_offset is not None:
            flags |= (1 << 0)
        buf.write(struct.pack('<I', flags))
        buf.write(struct.pack('<q', self.hash))
        buf.write(struct.pack('<i', 0x1cb5c415))  # vector id
        buf.write(struct.pack('<i', len(self.themes)))
        for item in self.themes:
            buf.write(bytes(self.item))
        buf.write(struct.pack('<i', 0x1cb5c415))  # vector id
        buf.write(struct.pack('<i', len(self.chats)))
        for item in self.chats:
            buf.write(bytes(self.item))
        buf.write(struct.pack('<i', 0x1cb5c415))  # vector id
        buf.write(struct.pack('<i', len(self.users)))
        for item in self.users:
            buf.write(bytes(self.item))
        if self.next_offset is not None:
            buf.write(TLObject.serialize_bytes(self.next_offset))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        flags = reader.read_int(signed=False)
        _val_hash = reader.read_long()
        _args['hash'] = _val_hash
        reader.read_int(signed=False)  # skip vector id
        _count_themes = reader.read_int()
        _list_themes = []
        for _ in range(_count_themes):
            _item_themes = reader.tgread_object()
            _list_themes.append(_item_themes)
        _args['themes'] = _list_themes
        reader.read_int(signed=False)  # skip vector id
        _count_chats = reader.read_int()
        _list_chats = []
        for _ in range(_count_chats):
            _item_chats = reader.tgread_object()
            _list_chats.append(_item_chats)
        _args['chats'] = _list_chats
        reader.read_int(signed=False)  # skip vector id
        _count_users = reader.read_int()
        _list_users = []
        for _ in range(_count_users):
            _item_users = reader.tgread_object()
            _list_users.append(_item_users)
        _args['users'] = _list_users
        if flags & (1 << 0):
            _val_next_offset = reader.tgread_string()
            _args['next_offset'] = _val_next_offset
        else:
            _args['next_offset'] = None
        return cls(**_args)


class ChatThemesNotModified(TLObject):
    """TL type: account.chatThemesNotModified"""
    CONSTRUCTOR_ID = 0xe011e1c4
    SUBCLASS_OF_ID = 0x15c14aa8

    def __init__(self):
        pass

    def to_dict(self):
        return {
            '_': 'ChatThemesNotModified',
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        return cls()


class ConnectedBots(TLObject):
    """TL type: account.connectedBots"""
    CONSTRUCTOR_ID = 0x17d7f87b
    SUBCLASS_OF_ID = 0xe4caf7d3

    def __init__(self, connected_bots: List['TypeConnectedBot'], users: List['TypeUser']):
        self.connected_bots = connected_bots
        self.users = users

    def to_dict(self):
        return {
            '_': 'ConnectedBots',
            'connected_bots': self.connected_bots,
            'users': self.users,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(struct.pack('<i', 0x1cb5c415))  # vector id
        buf.write(struct.pack('<i', len(self.connected_bots)))
        for item in self.connected_bots:
            buf.write(bytes(self.item))
        buf.write(struct.pack('<i', 0x1cb5c415))  # vector id
        buf.write(struct.pack('<i', len(self.users)))
        for item in self.users:
            buf.write(bytes(self.item))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        reader.read_int(signed=False)  # skip vector id
        _count_connected_bots = reader.read_int()
        _list_connected_bots = []
        for _ in range(_count_connected_bots):
            _item_connected_bots = reader.tgread_object()
            _list_connected_bots.append(_item_connected_bots)
        _args['connected_bots'] = _list_connected_bots
        reader.read_int(signed=False)  # skip vector id
        _count_users = reader.read_int()
        _list_users = []
        for _ in range(_count_users):
            _item_users = reader.tgread_object()
            _list_users.append(_item_users)
        _args['users'] = _list_users
        return cls(**_args)


class ContentSettings(TLObject):
    """TL type: account.contentSettings"""
    CONSTRUCTOR_ID = 0x57e28221
    SUBCLASS_OF_ID = 0xae3ff891

    def __init__(self, sensitive_enabled: Optional[bool] = None, sensitive_can_change: Optional[bool] = None):
        self.sensitive_enabled = sensitive_enabled
        self.sensitive_can_change = sensitive_can_change

    def to_dict(self):
        return {
            '_': 'ContentSettings',
            'sensitive_enabled': self.sensitive_enabled,
            'sensitive_can_change': self.sensitive_can_change,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        flags = 0
        if self.sensitive_enabled:
            flags |= (1 << 0)
        if self.sensitive_can_change:
            flags |= (1 << 1)
        buf.write(struct.pack('<I', flags))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        flags = reader.read_int(signed=False)
        _args['sensitive_enabled'] = bool(flags & (1 << 0))
        _args['sensitive_can_change'] = bool(flags & (1 << 1))
        return cls(**_args)


class EmailVerified(TLObject):
    """TL type: account.emailVerified"""
    CONSTRUCTOR_ID = 0x2b96cd1b
    SUBCLASS_OF_ID = 0x64833188

    def __init__(self, email: str):
        self.email = email

    def to_dict(self):
        return {
            '_': 'EmailVerified',
            'email': self.email,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(TLObject.serialize_bytes(self.email))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_email = reader.tgread_string()
        _args['email'] = _val_email
        return cls(**_args)


class EmailVerifiedLogin(TLObject):
    """TL type: account.emailVerifiedLogin"""
    CONSTRUCTOR_ID = 0xe1bb0d61
    SUBCLASS_OF_ID = 0x64833188

    def __init__(self, email: str, sent_code: 'TypeSentCode'):
        self.email = email
        self.sent_code = sent_code

    def to_dict(self):
        return {
            '_': 'EmailVerifiedLogin',
            'email': self.email,
            'sent_code': self.sent_code,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(TLObject.serialize_bytes(self.email))
        buf.write(bytes(self.sent_code))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_email = reader.tgread_string()
        _args['email'] = _val_email
        _val_sent_code = reader.tgread_object()
        _args['sent_code'] = _val_sent_code
        return cls(**_args)


class EmojiStatuses(TLObject):
    """TL type: account.emojiStatuses"""
    CONSTRUCTOR_ID = 0x90c467d1
    SUBCLASS_OF_ID = 0xd3e005ca

    def __init__(self, hash: int, statuses: List['TypeEmojiStatus']):
        self.hash = hash
        self.statuses = statuses

    def to_dict(self):
        return {
            '_': 'EmojiStatuses',
            'hash': self.hash,
            'statuses': self.statuses,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(struct.pack('<q', self.hash))
        buf.write(struct.pack('<i', 0x1cb5c415))  # vector id
        buf.write(struct.pack('<i', len(self.statuses)))
        for item in self.statuses:
            buf.write(bytes(self.item))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_hash = reader.read_long()
        _args['hash'] = _val_hash
        reader.read_int(signed=False)  # skip vector id
        _count_statuses = reader.read_int()
        _list_statuses = []
        for _ in range(_count_statuses):
            _item_statuses = reader.tgread_object()
            _list_statuses.append(_item_statuses)
        _args['statuses'] = _list_statuses
        return cls(**_args)


class EmojiStatusesNotModified(TLObject):
    """TL type: account.emojiStatusesNotModified"""
    CONSTRUCTOR_ID = 0xd08ce645
    SUBCLASS_OF_ID = 0xd3e005ca

    def __init__(self):
        pass

    def to_dict(self):
        return {
            '_': 'EmojiStatusesNotModified',
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        return cls()


class PaidMessagesRevenue(TLObject):
    """TL type: account.paidMessagesRevenue"""
    CONSTRUCTOR_ID = 0x1e109708
    SUBCLASS_OF_ID = 0x152f0c57

    def __init__(self, stars_amount: int):
        self.stars_amount = stars_amount

    def to_dict(self):
        return {
            '_': 'PaidMessagesRevenue',
            'stars_amount': self.stars_amount,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(struct.pack('<q', self.stars_amount))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_stars_amount = reader.read_long()
        _args['stars_amount'] = _val_stars_amount
        return cls(**_args)


class PasskeyRegistrationOptions(TLObject):
    """TL type: account.passkeyRegistrationOptions"""
    CONSTRUCTOR_ID = 0xe16b5ce1
    SUBCLASS_OF_ID = 0x341d83e4

    def __init__(self, options: 'TypeDataJSON'):
        self.options = options

    def to_dict(self):
        return {
            '_': 'PasskeyRegistrationOptions',
            'options': self.options,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(bytes(self.options))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_options = reader.tgread_object()
        _args['options'] = _val_options
        return cls(**_args)


class Passkeys(TLObject):
    """TL type: account.passkeys"""
    CONSTRUCTOR_ID = 0xf8e0aa1c
    SUBCLASS_OF_ID = 0x24dd205e

    def __init__(self, passkeys: List['TypePasskey']):
        self.passkeys = passkeys

    def to_dict(self):
        return {
            '_': 'Passkeys',
            'passkeys': self.passkeys,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(struct.pack('<i', 0x1cb5c415))  # vector id
        buf.write(struct.pack('<i', len(self.passkeys)))
        for item in self.passkeys:
            buf.write(bytes(self.item))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        reader.read_int(signed=False)  # skip vector id
        _count_passkeys = reader.read_int()
        _list_passkeys = []
        for _ in range(_count_passkeys):
            _item_passkeys = reader.tgread_object()
            _list_passkeys.append(_item_passkeys)
        _args['passkeys'] = _list_passkeys
        return cls(**_args)


class Password(TLObject):
    """TL type: account.password"""
    CONSTRUCTOR_ID = 0x957b50fb
    SUBCLASS_OF_ID = 0x53a211a3

    def __init__(self, new_algo: 'TypePasswordKdfAlgo', new_secure_algo: 'TypeSecurePasswordKdfAlgo', secure_random: bytes, has_recovery: Optional[bool] = None, has_secure_values: Optional[bool] = None, has_password: Optional[bool] = None, current_algo: Optional['TypePasswordKdfAlgo'] = None, srp_B: Optional[bytes] = None, srp_id: Optional[int] = None, hint: Optional[str] = None, email_unconfirmed_pattern: Optional[str] = None, pending_reset_date: Optional[datetime] = None, login_email_pattern: Optional[str] = None):
        self.new_algo = new_algo
        self.new_secure_algo = new_secure_algo
        self.secure_random = secure_random
        self.has_recovery = has_recovery
        self.has_secure_values = has_secure_values
        self.has_password = has_password
        self.current_algo = current_algo
        self.srp_B = srp_B
        self.srp_id = srp_id
        self.hint = hint
        self.email_unconfirmed_pattern = email_unconfirmed_pattern
        self.pending_reset_date = pending_reset_date
        self.login_email_pattern = login_email_pattern

    def to_dict(self):
        return {
            '_': 'Password',
            'new_algo': self.new_algo,
            'new_secure_algo': self.new_secure_algo,
            'secure_random': self.secure_random,
            'has_recovery': self.has_recovery,
            'has_secure_values': self.has_secure_values,
            'has_password': self.has_password,
            'current_algo': self.current_algo,
            'srp_B': self.srp_B,
            'srp_id': self.srp_id,
            'hint': self.hint,
            'email_unconfirmed_pattern': self.email_unconfirmed_pattern,
            'pending_reset_date': self.pending_reset_date,
            'login_email_pattern': self.login_email_pattern,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        flags = 0
        if self.has_recovery:
            flags |= (1 << 0)
        if self.has_secure_values:
            flags |= (1 << 1)
        if self.srp_id is not None:
            flags |= (1 << 2)
        if self.hint is not None:
            flags |= (1 << 3)
        if self.email_unconfirmed_pattern is not None:
            flags |= (1 << 4)
        if self.pending_reset_date is not None:
            flags |= (1 << 5)
        if self.login_email_pattern is not None:
            flags |= (1 << 6)
        buf.write(struct.pack('<I', flags))
        buf.write(bytes(self.new_algo))
        buf.write(bytes(self.new_secure_algo))
        buf.write(TLObject.serialize_bytes(self.secure_random))
        if self.current_algo is not None:
            buf.write(bytes(self.current_algo))
        if self.srp_B is not None:
            buf.write(TLObject.serialize_bytes(self.srp_B))
        if self.srp_id is not None:
            buf.write(struct.pack('<q', self.srp_id))
        if self.hint is not None:
            buf.write(TLObject.serialize_bytes(self.hint))
        if self.email_unconfirmed_pattern is not None:
            buf.write(TLObject.serialize_bytes(self.email_unconfirmed_pattern))
        if self.pending_reset_date is not None:
            buf.write(struct.pack('<i', self.pending_reset_date))
        if self.login_email_pattern is not None:
            buf.write(TLObject.serialize_bytes(self.login_email_pattern))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        flags = reader.read_int(signed=False)
        _args['has_recovery'] = bool(flags & (1 << 0))
        _args['has_secure_values'] = bool(flags & (1 << 1))
        _args['has_password'] = bool(flags & (1 << 2))
        if flags & (1 << 2):
            _val_current_algo = reader.tgread_object()
            _args['current_algo'] = _val_current_algo
        else:
            _args['current_algo'] = None
        if flags & (1 << 2):
            _val_srp_B = reader.tgread_bytes()
            _args['srp_B'] = _val_srp_B
        else:
            _args['srp_B'] = None
        if flags & (1 << 2):
            _val_srp_id = reader.read_long()
            _args['srp_id'] = _val_srp_id
        else:
            _args['srp_id'] = None
        if flags & (1 << 3):
            _val_hint = reader.tgread_string()
            _args['hint'] = _val_hint
        else:
            _args['hint'] = None
        if flags & (1 << 4):
            _val_email_unconfirmed_pattern = reader.tgread_string()
            _args['email_unconfirmed_pattern'] = _val_email_unconfirmed_pattern
        else:
            _args['email_unconfirmed_pattern'] = None
        _val_new_algo = reader.tgread_object()
        _args['new_algo'] = _val_new_algo
        _val_new_secure_algo = reader.tgread_object()
        _args['new_secure_algo'] = _val_new_secure_algo
        _val_secure_random = reader.tgread_bytes()
        _args['secure_random'] = _val_secure_random
        if flags & (1 << 5):
            _val_pending_reset_date = reader.read_int()
            _args['pending_reset_date'] = _val_pending_reset_date
        else:
            _args['pending_reset_date'] = None
        if flags & (1 << 6):
            _val_login_email_pattern = reader.tgread_string()
            _args['login_email_pattern'] = _val_login_email_pattern
        else:
            _args['login_email_pattern'] = None
        return cls(**_args)


class PasswordInputSettings(TLObject):
    """TL type: account.passwordInputSettings"""
    CONSTRUCTOR_ID = 0xc23727c9
    SUBCLASS_OF_ID = 0x0c426ca6

    def __init__(self, new_algo: Optional['TypePasswordKdfAlgo'] = None, new_password_hash: Optional[bytes] = None, hint: Optional[str] = None, email: Optional[str] = None, new_secure_settings: Optional['TypeSecureSecretSettings'] = None):
        self.new_algo = new_algo
        self.new_password_hash = new_password_hash
        self.hint = hint
        self.email = email
        self.new_secure_settings = new_secure_settings

    def to_dict(self):
        return {
            '_': 'PasswordInputSettings',
            'new_algo': self.new_algo,
            'new_password_hash': self.new_password_hash,
            'hint': self.hint,
            'email': self.email,
            'new_secure_settings': self.new_secure_settings,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        flags = 0
        if self.hint is not None:
            flags |= (1 << 0)
        if self.email is not None:
            flags |= (1 << 1)
        if self.new_secure_settings is not None:
            flags |= (1 << 2)
        buf.write(struct.pack('<I', flags))
        if self.new_algo is not None:
            buf.write(bytes(self.new_algo))
        if self.new_password_hash is not None:
            buf.write(TLObject.serialize_bytes(self.new_password_hash))
        if self.hint is not None:
            buf.write(TLObject.serialize_bytes(self.hint))
        if self.email is not None:
            buf.write(TLObject.serialize_bytes(self.email))
        if self.new_secure_settings is not None:
            buf.write(bytes(self.new_secure_settings))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        flags = reader.read_int(signed=False)
        if flags & (1 << 0):
            _val_new_algo = reader.tgread_object()
            _args['new_algo'] = _val_new_algo
        else:
            _args['new_algo'] = None
        if flags & (1 << 0):
            _val_new_password_hash = reader.tgread_bytes()
            _args['new_password_hash'] = _val_new_password_hash
        else:
            _args['new_password_hash'] = None
        if flags & (1 << 0):
            _val_hint = reader.tgread_string()
            _args['hint'] = _val_hint
        else:
            _args['hint'] = None
        if flags & (1 << 1):
            _val_email = reader.tgread_string()
            _args['email'] = _val_email
        else:
            _args['email'] = None
        if flags & (1 << 2):
            _val_new_secure_settings = reader.tgread_object()
            _args['new_secure_settings'] = _val_new_secure_settings
        else:
            _args['new_secure_settings'] = None
        return cls(**_args)


class PasswordSettings(TLObject):
    """TL type: account.passwordSettings"""
    CONSTRUCTOR_ID = 0x9a5c33e5
    SUBCLASS_OF_ID = 0xd23fb078

    def __init__(self, email: Optional[str] = None, secure_settings: Optional['TypeSecureSecretSettings'] = None):
        self.email = email
        self.secure_settings = secure_settings

    def to_dict(self):
        return {
            '_': 'PasswordSettings',
            'email': self.email,
            'secure_settings': self.secure_settings,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        flags = 0
        if self.email is not None:
            flags |= (1 << 0)
        if self.secure_settings is not None:
            flags |= (1 << 1)
        buf.write(struct.pack('<I', flags))
        if self.email is not None:
            buf.write(TLObject.serialize_bytes(self.email))
        if self.secure_settings is not None:
            buf.write(bytes(self.secure_settings))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        flags = reader.read_int(signed=False)
        if flags & (1 << 0):
            _val_email = reader.tgread_string()
            _args['email'] = _val_email
        else:
            _args['email'] = None
        if flags & (1 << 1):
            _val_secure_settings = reader.tgread_object()
            _args['secure_settings'] = _val_secure_settings
        else:
            _args['secure_settings'] = None
        return cls(**_args)


class PrivacyRules(TLObject):
    """TL type: account.privacyRules"""
    CONSTRUCTOR_ID = 0x50a04e45
    SUBCLASS_OF_ID = 0xb55aba82

    def __init__(self, rules: List['TypePrivacyRule'], chats: List['TypeChat'], users: List['TypeUser']):
        self.rules = rules
        self.chats = chats
        self.users = users

    def to_dict(self):
        return {
            '_': 'PrivacyRules',
            'rules': self.rules,
            'chats': self.chats,
            'users': self.users,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(struct.pack('<i', 0x1cb5c415))  # vector id
        buf.write(struct.pack('<i', len(self.rules)))
        for item in self.rules:
            buf.write(bytes(self.item))
        buf.write(struct.pack('<i', 0x1cb5c415))  # vector id
        buf.write(struct.pack('<i', len(self.chats)))
        for item in self.chats:
            buf.write(bytes(self.item))
        buf.write(struct.pack('<i', 0x1cb5c415))  # vector id
        buf.write(struct.pack('<i', len(self.users)))
        for item in self.users:
            buf.write(bytes(self.item))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        reader.read_int(signed=False)  # skip vector id
        _count_rules = reader.read_int()
        _list_rules = []
        for _ in range(_count_rules):
            _item_rules = reader.tgread_object()
            _list_rules.append(_item_rules)
        _args['rules'] = _list_rules
        reader.read_int(signed=False)  # skip vector id
        _count_chats = reader.read_int()
        _list_chats = []
        for _ in range(_count_chats):
            _item_chats = reader.tgread_object()
            _list_chats.append(_item_chats)
        _args['chats'] = _list_chats
        reader.read_int(signed=False)  # skip vector id
        _count_users = reader.read_int()
        _list_users = []
        for _ in range(_count_users):
            _item_users = reader.tgread_object()
            _list_users.append(_item_users)
        _args['users'] = _list_users
        return cls(**_args)


class ResetPasswordFailedWait(TLObject):
    """TL type: account.resetPasswordFailedWait"""
    CONSTRUCTOR_ID = 0xe3779861
    SUBCLASS_OF_ID = 0x49507416

    def __init__(self, retry_date: Optional[datetime]):
        self.retry_date = retry_date

    def to_dict(self):
        return {
            '_': 'ResetPasswordFailedWait',
            'retry_date': self.retry_date,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(struct.pack('<i', self.retry_date))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_retry_date = reader.read_int()
        _args['retry_date'] = _val_retry_date
        return cls(**_args)


class ResetPasswordOk(TLObject):
    """TL type: account.resetPasswordOk"""
    CONSTRUCTOR_ID = 0xe926d63e
    SUBCLASS_OF_ID = 0x49507416

    def __init__(self):
        pass

    def to_dict(self):
        return {
            '_': 'ResetPasswordOk',
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        return cls()


class ResetPasswordRequestedWait(TLObject):
    """TL type: account.resetPasswordRequestedWait"""
    CONSTRUCTOR_ID = 0xe9effc7d
    SUBCLASS_OF_ID = 0x49507416

    def __init__(self, until_date: Optional[datetime]):
        self.until_date = until_date

    def to_dict(self):
        return {
            '_': 'ResetPasswordRequestedWait',
            'until_date': self.until_date,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(struct.pack('<i', self.until_date))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_until_date = reader.read_int()
        _args['until_date'] = _val_until_date
        return cls(**_args)


class ResolvedBusinessChatLinks(TLObject):
    """TL type: account.resolvedBusinessChatLinks"""
    CONSTRUCTOR_ID = 0x9a23af21
    SUBCLASS_OF_ID = 0x3a772c28

    def __init__(self, peer: 'TypePeer', message: str, chats: List['TypeChat'], users: List['TypeUser'], entities: Optional[List['TypeMessageEntity']] = None):
        self.peer = peer
        self.message = message
        self.chats = chats
        self.users = users
        self.entities = entities

    def to_dict(self):
        return {
            '_': 'ResolvedBusinessChatLinks',
            'peer': self.peer,
            'message': self.message,
            'chats': self.chats,
            'users': self.users,
            'entities': self.entities,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        flags = 0
        if self.entities is not None:
            flags |= (1 << 0)
        buf.write(struct.pack('<I', flags))
        buf.write(bytes(self.peer))
        buf.write(TLObject.serialize_bytes(self.message))
        buf.write(struct.pack('<i', 0x1cb5c415))  # vector id
        buf.write(struct.pack('<i', len(self.chats)))
        for item in self.chats:
            buf.write(bytes(self.item))
        buf.write(struct.pack('<i', 0x1cb5c415))  # vector id
        buf.write(struct.pack('<i', len(self.users)))
        for item in self.users:
            buf.write(bytes(self.item))
        if self.entities is not None:
            buf.write(struct.pack('<i', 0x1cb5c415))  # vector id
            buf.write(struct.pack('<i', len(self.entities)))
            for item in self.entities:
                buf.write(bytes(self.item))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        flags = reader.read_int(signed=False)
        _val_peer = reader.tgread_object()
        _args['peer'] = _val_peer
        _val_message = reader.tgread_string()
        _args['message'] = _val_message
        if flags & (1 << 0):
            reader.read_int(signed=False)  # skip vector id
            _count_entities = reader.read_int()
            _list_entities = []
            for _ in range(_count_entities):
                _item_entities = reader.tgread_object()
                _list_entities.append(_item_entities)
            _args['entities'] = _list_entities
        else:
            _args['entities'] = None
        reader.read_int(signed=False)  # skip vector id
        _count_chats = reader.read_int()
        _list_chats = []
        for _ in range(_count_chats):
            _item_chats = reader.tgread_object()
            _list_chats.append(_item_chats)
        _args['chats'] = _list_chats
        reader.read_int(signed=False)  # skip vector id
        _count_users = reader.read_int()
        _list_users = []
        for _ in range(_count_users):
            _item_users = reader.tgread_object()
            _list_users.append(_item_users)
        _args['users'] = _list_users
        return cls(**_args)


class SavedMusicIds(TLObject):
    """TL type: account.savedMusicIds"""
    CONSTRUCTOR_ID = 0x998d6636
    SUBCLASS_OF_ID = 0x4b4af692

    def __init__(self, ids: List[int]):
        self.ids = ids

    def to_dict(self):
        return {
            '_': 'SavedMusicIds',
            'ids': self.ids,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(struct.pack('<i', 0x1cb5c415))  # vector id
        buf.write(struct.pack('<i', len(self.ids)))
        for item in self.ids:
            buf.write(struct.pack('<q', self.item))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        reader.read_int(signed=False)  # skip vector id
        _count_ids = reader.read_int()
        _list_ids = []
        for _ in range(_count_ids):
            _item_ids = reader.read_long()
            _list_ids.append(_item_ids)
        _args['ids'] = _list_ids
        return cls(**_args)


class SavedMusicIdsNotModified(TLObject):
    """TL type: account.savedMusicIdsNotModified"""
    CONSTRUCTOR_ID = 0x4fc81d6e
    SUBCLASS_OF_ID = 0x4b4af692

    def __init__(self):
        pass

    def to_dict(self):
        return {
            '_': 'SavedMusicIdsNotModified',
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        return cls()


class SavedRingtone(TLObject):
    """TL type: account.savedRingtone"""
    CONSTRUCTOR_ID = 0xb7263f6d
    SUBCLASS_OF_ID = 0xb1e28424

    def __init__(self):
        pass

    def to_dict(self):
        return {
            '_': 'SavedRingtone',
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        return cls()


class SavedRingtoneConverted(TLObject):
    """TL type: account.savedRingtoneConverted"""
    CONSTRUCTOR_ID = 0x1f307eb7
    SUBCLASS_OF_ID = 0xb1e28424

    def __init__(self, document: 'TypeDocument'):
        self.document = document

    def to_dict(self):
        return {
            '_': 'SavedRingtoneConverted',
            'document': self.document,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(bytes(self.document))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_document = reader.tgread_object()
        _args['document'] = _val_document
        return cls(**_args)


class SavedRingtones(TLObject):
    """TL type: account.savedRingtones"""
    CONSTRUCTOR_ID = 0xc1e92cc5
    SUBCLASS_OF_ID = 0x27bcc95e

    def __init__(self, hash: int, ringtones: List['TypeDocument']):
        self.hash = hash
        self.ringtones = ringtones

    def to_dict(self):
        return {
            '_': 'SavedRingtones',
            'hash': self.hash,
            'ringtones': self.ringtones,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(struct.pack('<q', self.hash))
        buf.write(struct.pack('<i', 0x1cb5c415))  # vector id
        buf.write(struct.pack('<i', len(self.ringtones)))
        for item in self.ringtones:
            buf.write(bytes(self.item))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_hash = reader.read_long()
        _args['hash'] = _val_hash
        reader.read_int(signed=False)  # skip vector id
        _count_ringtones = reader.read_int()
        _list_ringtones = []
        for _ in range(_count_ringtones):
            _item_ringtones = reader.tgread_object()
            _list_ringtones.append(_item_ringtones)
        _args['ringtones'] = _list_ringtones
        return cls(**_args)


class SavedRingtonesNotModified(TLObject):
    """TL type: account.savedRingtonesNotModified"""
    CONSTRUCTOR_ID = 0xfbf6e8b1
    SUBCLASS_OF_ID = 0x27bcc95e

    def __init__(self):
        pass

    def to_dict(self):
        return {
            '_': 'SavedRingtonesNotModified',
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        return cls()


class SentEmailCode(TLObject):
    """TL type: account.sentEmailCode"""
    CONSTRUCTOR_ID = 0x811f854f
    SUBCLASS_OF_ID = 0x69f3c06e

    def __init__(self, email_pattern: str, length: int):
        self.email_pattern = email_pattern
        self.length = length

    def to_dict(self):
        return {
            '_': 'SentEmailCode',
            'email_pattern': self.email_pattern,
            'length': self.length,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(TLObject.serialize_bytes(self.email_pattern))
        buf.write(struct.pack('<i', self.length))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_email_pattern = reader.tgread_string()
        _args['email_pattern'] = _val_email_pattern
        _val_length = reader.read_int()
        _args['length'] = _val_length
        return cls(**_args)


class Takeout(TLObject):
    """TL type: account.takeout"""
    CONSTRUCTOR_ID = 0x4dba4501
    SUBCLASS_OF_ID = 0x843ebe85

    def __init__(self, id: int):
        self.id = id

    def to_dict(self):
        return {
            '_': 'Takeout',
            'id': self.id,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(struct.pack('<q', self.id))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_id = reader.read_long()
        _args['id'] = _val_id
        return cls(**_args)


class Themes(TLObject):
    """TL type: account.themes"""
    CONSTRUCTOR_ID = 0x9a3d8c6d
    SUBCLASS_OF_ID = 0x7fc52204

    def __init__(self, hash: int, themes: List['TypeTheme']):
        self.hash = hash
        self.themes = themes

    def to_dict(self):
        return {
            '_': 'Themes',
            'hash': self.hash,
            'themes': self.themes,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(struct.pack('<q', self.hash))
        buf.write(struct.pack('<i', 0x1cb5c415))  # vector id
        buf.write(struct.pack('<i', len(self.themes)))
        for item in self.themes:
            buf.write(bytes(self.item))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_hash = reader.read_long()
        _args['hash'] = _val_hash
        reader.read_int(signed=False)  # skip vector id
        _count_themes = reader.read_int()
        _list_themes = []
        for _ in range(_count_themes):
            _item_themes = reader.tgread_object()
            _list_themes.append(_item_themes)
        _args['themes'] = _list_themes
        return cls(**_args)


class ThemesNotModified(TLObject):
    """TL type: account.themesNotModified"""
    CONSTRUCTOR_ID = 0xf41eb622
    SUBCLASS_OF_ID = 0x7fc52204

    def __init__(self):
        pass

    def to_dict(self):
        return {
            '_': 'ThemesNotModified',
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        return cls()


class TmpPassword(TLObject):
    """TL type: account.tmpPassword"""
    CONSTRUCTOR_ID = 0xdb64fd34
    SUBCLASS_OF_ID = 0xb064992d

    def __init__(self, tmp_password: bytes, valid_until: Optional[datetime]):
        self.tmp_password = tmp_password
        self.valid_until = valid_until

    def to_dict(self):
        return {
            '_': 'TmpPassword',
            'tmp_password': self.tmp_password,
            'valid_until': self.valid_until,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(TLObject.serialize_bytes(self.tmp_password))
        buf.write(struct.pack('<i', self.valid_until))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_tmp_password = reader.tgread_bytes()
        _args['tmp_password'] = _val_tmp_password
        _val_valid_until = reader.read_int()
        _args['valid_until'] = _val_valid_until
        return cls(**_args)


class WallPapers(TLObject):
    """TL type: account.wallPapers"""
    CONSTRUCTOR_ID = 0xcdc3858c
    SUBCLASS_OF_ID = 0xa2c548fd

    def __init__(self, hash: int, wallpapers: List['TypeWallPaper']):
        self.hash = hash
        self.wallpapers = wallpapers

    def to_dict(self):
        return {
            '_': 'WallPapers',
            'hash': self.hash,
            'wallpapers': self.wallpapers,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(struct.pack('<q', self.hash))
        buf.write(struct.pack('<i', 0x1cb5c415))  # vector id
        buf.write(struct.pack('<i', len(self.wallpapers)))
        for item in self.wallpapers:
            buf.write(bytes(self.item))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_hash = reader.read_long()
        _args['hash'] = _val_hash
        reader.read_int(signed=False)  # skip vector id
        _count_wallpapers = reader.read_int()
        _list_wallpapers = []
        for _ in range(_count_wallpapers):
            _item_wallpapers = reader.tgread_object()
            _list_wallpapers.append(_item_wallpapers)
        _args['wallpapers'] = _list_wallpapers
        return cls(**_args)


class WallPapersNotModified(TLObject):
    """TL type: account.wallPapersNotModified"""
    CONSTRUCTOR_ID = 0x1c199183
    SUBCLASS_OF_ID = 0xa2c548fd

    def __init__(self):
        pass

    def to_dict(self):
        return {
            '_': 'WallPapersNotModified',
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        return cls()


class WebAuthorizations(TLObject):
    """TL type: account.webAuthorizations"""
    CONSTRUCTOR_ID = 0xed56c9fc
    SUBCLASS_OF_ID = 0x9a365b32

    def __init__(self, authorizations: List['TypeWebAuthorization'], users: List['TypeUser']):
        self.authorizations = authorizations
        self.users = users

    def to_dict(self):
        return {
            '_': 'WebAuthorizations',
            'authorizations': self.authorizations,
            'users': self.users,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(struct.pack('<i', 0x1cb5c415))  # vector id
        buf.write(struct.pack('<i', len(self.authorizations)))
        for item in self.authorizations:
            buf.write(bytes(self.item))
        buf.write(struct.pack('<i', 0x1cb5c415))  # vector id
        buf.write(struct.pack('<i', len(self.users)))
        for item in self.users:
            buf.write(bytes(self.item))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        reader.read_int(signed=False)  # skip vector id
        _count_authorizations = reader.read_int()
        _list_authorizations = []
        for _ in range(_count_authorizations):
            _item_authorizations = reader.tgread_object()
            _list_authorizations.append(_item_authorizations)
        _args['authorizations'] = _list_authorizations
        reader.read_int(signed=False)  # skip vector id
        _count_users = reader.read_int()
        _list_users = []
        for _ in range(_count_users):
            _item_users = reader.tgread_object()
            _list_users.append(_item_users)
        _args['users'] = _list_users
        return cls(**_args)




"""
MTProtoSender - encrypted MTProto 2.0 sender with send/receive loops.
Handles reconnection, retries, and all protocol-level responses.
"""
import asyncio
import logging
import struct
import time

from . import authenticator
from .mtprotoplainsender import MTProtoPlainSender
from .mtprotostate import MTProtoState
from .requeststate import RequestState
from ..crypto import AuthKey
from ..errors import (
    BadMessageError, InvalidBufferError, SecurityError,
    TypeNotFoundError, rpc_message_to_error, AuthKeyNotFound
)
from ..extensions import BinaryReader, MessagePacker
from ..helpers import is_list_like, retry_range
from ..tl.core import RpcResult, MessageContainer, GzipPacked
from ..tl.core.tlmessage import TLMessage

__log__ = logging.getLogger(__name__)


class MTProtoSender:
    """
    Encrypted MTProto sender with automatic auth key generation,
    reconnection, and full protocol handler dispatch.
    """

    def __init__(self, auth_key=None, *, loggers=None,
                 retries=5, delay=1.0, auto_reconnect=True,
                 connect_timeout=10.0,
                 auth_key_callback=None,
                 updates_queue=None):
        self._loggers = loggers or {__name__: __log__}
        self._log = self._loggers.get(__name__, __log__)
        self._retries = retries
        self._delay = delay
        self._auto_reconnect = auto_reconnect
        self._connect_timeout = connect_timeout
        self._auth_key_callback = auth_key_callback
        self._updates_queue = updates_queue
        self._connect_lock = asyncio.Lock()

        self._connection = None
        self._user_connected = False
        self._reconnecting = False
        self._disconnected = asyncio.get_event_loop().create_future()

        self.auth_key = auth_key or AuthKey(None)
        self._state = MTProtoState(self.auth_key, loggers=self._loggers)

        self._send_queue = MessagePacker(self._state, loggers=self._loggers)
        self._pending_state = {}
        self._pending_ack = set()

        self._send_loop_handle = None
        self._recv_loop_handle = None

        # Handler dispatch table
        self._handlers = {
            RpcResult.CONSTRUCTOR_ID:       self._handle_rpc_result,
            MessageContainer.CONSTRUCTOR_ID: self._handle_container,
            GzipPacked.CONSTRUCTOR_ID:      self._handle_gzip_packed,
            0x347773c5: self._handle_pong,              # Pong
            0xedab447b: self._handle_bad_server_salt,   # BadServerSalt
            0xa7eff811: self._handle_bad_notification,  # BadMsgNotification
            0x9ec20908: self._handle_new_session_created, # NewSessionCreated
            0x62d6b459: self._handle_ack,               # MsgsAck
            0x276d3ec6: self._handle_msg_detailed_info, # MsgDetailedInfo
            0x809db6df: self._handle_msg_detailed_info, # MsgNewDetailedInfo
        }

    # ── Public API ─────────────────────────────────────────────────────────

    async def connect(self, connection):
        async with self._connect_lock:
            if self._user_connected:
                return False
            self._connection = connection
            await self._connect()
            self._user_connected = True
            return True

    async def disconnect(self):
        await self._disconnect()

    def send(self, request, ordered=False):
        """
        Queue a request for sending.
        Returns an asyncio.Future that resolves with the response.
        """
        if not self._user_connected:
            raise ConnectionError('Cannot send while disconnected')

        if not is_list_like(request):
            state = RequestState(request)
            self._send_queue.append(state)
            return state.future
        else:
            states = []
            prev = None
            for req in request:
                state = RequestState(req, after=prev if ordered else None)
                states.append(state)
                prev = state
                self._send_queue.append(state)
            return [s.future for s in states]

    def is_connected(self) -> bool:
        return self._user_connected

    # ── Internal connection management ────────────────────────────────────

    async def _connect(self):
        await self._connection.connect()

        # Generate auth key if needed
        if not self.auth_key:
            self._log.info('No auth key — generating...')
            plain = MTProtoPlainSender(self._connection, loggers=self._loggers)
            self.auth_key, self._state.time_offset = \
                await authenticator.do_authentication(plain)
            self._state.auth_key = self.auth_key
            if self._auth_key_callback:
                await self._auth_key_callback(self.auth_key)
            self._log.info('Auth key generated (key_id=%#018x)',
                           self.auth_key.key_id)
        else:
            self._state.auth_key = self.auth_key

        self._state.reset()
        self._send_loop_handle = asyncio.ensure_future(self._send_loop())
        self._recv_loop_handle = asyncio.ensure_future(self._recv_loop())

    async def _disconnect(self):
        self._user_connected = False
        self._send_queue.put_nowait(None)
        if self._send_loop_handle:
            self._send_loop_handle.cancel()
        if self._recv_loop_handle:
            self._recv_loop_handle.cancel()
        await self._connection.disconnect()
        # Cancel pending futures
        for state in self._pending_state.values():
            if not state.future.done():
                state.future.cancel()
        self._pending_state.clear()

    # ── Send/Receive loops ────────────────────────────────────────────────

    async def _send_loop(self):
        """Continuously pack and send pending requests."""
        while self._user_connected:
            try:
                data, states = await self._send_queue.get()
                if data is None:
                    break

                # Add any pending acks
                if self._pending_ack:
                    self._send_ack(self._pending_ack)
                    self._pending_ack.clear()

                if states:
                    for state in states:
                        if state.msg_id:
                            self._pending_state[state.msg_id] = state

                encrypted = self._state.encrypt_message_data(data)
                await self._connection.send(encrypted)

            except asyncio.CancelledError:
                break
            except Exception as e:
                self._log.error('Send error: %s', e)
                if self._auto_reconnect:
                    await self._try_reconnect()
                return

    async def _recv_loop(self):
        """Continuously receive and dispatch incoming messages."""
        while self._user_connected:
            try:
                body = await self._connection.recv()
                message = self._state.decrypt_message_data(body)
                if message:
                    await self._process_message(message)
            except asyncio.CancelledError:
                break
            except InvalidBufferError as e:
                self._log.warning('Invalid buffer: %s', e)
            except SecurityError as e:
                self._log.error('Security error: %s', e)
                break
            except TypeNotFoundError as e:
                # Telegram server is on a newer layer than our TL schema.
                # The enclosing MTProto frame is already consumed so stream
                # sync is preserved.  Log at DEBUG to keep output clean.
                self._log.debug(
                    'Skipping unknown TL constructor 0x%08x '
                    '(schema needs update for Telegram layer 225+)',
                    e.invalid_constructor_id
                )
            except Exception as e:
                self._log.error('Recv error: %s', e)
                if self._auto_reconnect:
                    await self._try_reconnect()
                return

    async def _try_reconnect(self):
        if self._reconnecting:
            return
        self._reconnecting = True
        self._log.info('Reconnecting...')
        try:
            async for attempt in retry_range(self._retries, delay=self._delay):
                try:
                    await self._connection.disconnect()
                    await self._connection.connect()
                    self._reconnecting = False
                    self._log.info('Reconnected')
                    # Restart both loops so the connection is fully usable again
                    self._send_loop_handle = asyncio.ensure_future(self._send_loop())
                    self._recv_loop_handle = asyncio.ensure_future(self._recv_loop())
                    return
                except Exception as e:
                    self._log.warning('Reconnect attempt %d failed: %s', attempt + 1, e)
        except Exception:
            pass
        self._reconnecting = False
        self._user_connected = False

    # ── Protocol message handlers ─────────────────────────────────────────

    async def _process_message(self, message: TLMessage):
        handler = self._handlers.get(message.obj.CONSTRUCTOR_ID)
        if handler:
            await handler(message)
        elif self._updates_queue:
            await self._updates_queue.put(message.obj)

    async def _handle_rpc_result(self, message: TLMessage):
        rpc = message.obj
        state = self._pending_state.pop(rpc.req_msg_id, None)
        self._pending_ack.add(message.msg_id)

        if state is None:
            self._log.warning('RPC result for unknown msg_id %d', rpc.req_msg_id)
            return

        if rpc.error:
            code, msg = rpc.error
            error = rpc_message_to_error(code, msg, state.request)
            if not state.future.done():
                state.future.set_exception(error)
        else:
            if not state.future.done():
                state.future.set_result(rpc.body)

    async def _handle_container(self, message: TLMessage):
        from ..errors import TypeNotFoundError
        for inner in message.obj.messages:
            try:
                await self._process_message(inner)
            except TypeNotFoundError as e:
                self._log.debug(
                    'Skipping container sub-message with unknown TL '
                    'constructor 0x%08x (schema needs update)',
                    e.invalid_constructor_id
                )

    async def _handle_gzip_packed(self, message: TLMessage):
        from ..extensions import BinaryReader
        reader = BinaryReader(message.obj.data)
        inner_obj = reader.tgread_object()
        await self._process_message(TLMessage(
            message.msg_id, message.seq_no, inner_obj
        ))

    async def _handle_pong(self, message: TLMessage):
        pong = message.obj
        state = self._pending_state.pop(pong.msg_id, None)
        if state and not state.future.done():
            state.future.set_result(pong)

    async def _handle_bad_server_salt(self, message: TLMessage):
        bad = message.obj
        self._state.salt = bad.new_server_salt
        state = self._pending_state.get(bad.bad_msg_id)
        if state:
            self._state.update_message_id(state)
            self._send_queue.append(state)

    async def _handle_bad_notification(self, message: TLMessage):
        bad = message.obj
        state = self._pending_state.pop(bad.bad_msg_id, None)
        if state and not state.future.done():
            state.future.set_exception(BadMessageError(bad.error_code))

    async def _handle_new_session_created(self, message: TLMessage):
        self._state.salt = message.obj.server_salt

    async def _handle_ack(self, message: TLMessage):
        pass  # Acknowledgements don't require action

    async def _handle_msg_detailed_info(self, message: TLMessage):
        # Server is telling us about a message we might have missed; request a resend
        self._pending_ack.add(message.obj.answer_msg_id)

    def _send_ack(self, msg_ids):
        from ..tl.mtproto_types import MsgsAck
        ack = MsgsAck(msg_ids=list(msg_ids))
        state = RequestState(ack)
        self._send_queue.append(state)



"""GzipPacked TL core type - re-exported from rpcresult for backward compatibility."""
import gzip
import struct

from ..tlobject import TLObject


class GzipPacked(TLObject):
    CONSTRUCTOR_ID = 0x3072cfa1

    def __init__(self, data: bytes):
        self.data = data  # already compressed bytes

    @staticmethod
    def gzip_if_smaller(content_related: bool, data: bytes) -> bytes:
        """Return gzip-packed data if smaller, else original data."""
        if content_related:
            try:
                compressed = gzip.compress(data, compresslevel=6)
                if len(compressed) < len(data):
                    packed = GzipPacked(compressed)
                    return bytes(packed)
            except Exception:
                pass
        return data

    def _bytes(self) -> bytes:
        return (
            struct.pack('<I', self.CONSTRUCTOR_ID) +
            TLObject.serialize_bytes(self.data)
        )

    @classmethod
    def from_reader(cls, reader):
        packed = reader.tgread_bytes()
        return cls(gzip.decompress(packed))

    def to_dict(self):
        return {'_': 'GzipPacked'}



"""File auto-generated by tglib TL generator. Do not edit manually."""
from ..tlobject import TLObject
from typing import Optional, List, Union
import os
import struct
from datetime import datetime


class CollectibleInfo(TLObject):
    """TL type: fragment.collectibleInfo"""
    CONSTRUCTOR_ID = 0x6ebdff91
    SUBCLASS_OF_ID = 0xd4ea5790

    def __init__(self, purchase_date: Optional[datetime], currency: str, amount: int, crypto_currency: str, crypto_amount: int, url: str):
        self.purchase_date = purchase_date
        self.currency = currency
        self.amount = amount
        self.crypto_currency = crypto_currency
        self.crypto_amount = crypto_amount
        self.url = url

    def to_dict(self):
        return {
            '_': 'CollectibleInfo',
            'purchase_date': self.purchase_date,
            'currency': self.currency,
            'amount': self.amount,
            'crypto_currency': self.crypto_currency,
            'crypto_amount': self.crypto_amount,
            'url': self.url,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(struct.pack('<i', self.purchase_date))
        buf.write(TLObject.serialize_bytes(self.currency))
        buf.write(struct.pack('<q', self.amount))
        buf.write(TLObject.serialize_bytes(self.crypto_currency))
        buf.write(struct.pack('<q', self.crypto_amount))
        buf.write(TLObject.serialize_bytes(self.url))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_purchase_date = reader.read_int()
        _args['purchase_date'] = _val_purchase_date
        _val_currency = reader.tgread_string()
        _args['currency'] = _val_currency
        _val_amount = reader.read_long()
        _args['amount'] = _val_amount
        _val_crypto_currency = reader.tgread_string()
        _args['crypto_currency'] = _val_crypto_currency
        _val_crypto_amount = reader.read_long()
        _args['crypto_amount'] = _val_crypto_amount
        _val_url = reader.tgread_string()
        _args['url'] = _val_url
        return cls(**_args)




"""TL (Type Language) module - types, functions, and core protocol objects."""
from . import types, functions, core
from .tlobject import TLObject, TLRequest

__all__ = ['types', 'functions', 'core', 'TLObject', 'TLRequest']



"""File auto-generated by tglib TL generator. Do not edit manually."""
from ..tlobject import TLObject
from ..tlobject import TLRequest
from typing import Optional, List, Union
import os
import struct
from datetime import datetime


class AcceptTermsOfServiceRequest(TLRequest):
    """TL type: help.acceptTermsOfService"""
    CONSTRUCTOR_ID = 0xee72f79a
    SUBCLASS_OF_ID = 0xf5b399ac

    def __init__(self, id: 'TypeDataJSON'):
        self.id = id

    def to_dict(self):
        return {
            '_': 'AcceptTermsOfServiceRequest',
            'id': self.id,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(bytes(self.id))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_id = reader.tgread_object()
        _args['id'] = _val_id
        return cls(**_args)


class DismissSuggestionRequest(TLRequest):
    """TL type: help.dismissSuggestion"""
    CONSTRUCTOR_ID = 0xf50dbaa1
    SUBCLASS_OF_ID = 0xf5b399ac

    def __init__(self, peer: 'TypeInputPeer', suggestion: str):
        self.peer = peer
        self.suggestion = suggestion

    def to_dict(self):
        return {
            '_': 'DismissSuggestionRequest',
            'peer': self.peer,
            'suggestion': self.suggestion,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(bytes(self.peer))
        buf.write(TLObject.serialize_bytes(self.suggestion))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_peer = reader.tgread_object()
        _args['peer'] = _val_peer
        _val_suggestion = reader.tgread_string()
        _args['suggestion'] = _val_suggestion
        return cls(**_args)


class EditUserInfoRequest(TLRequest):
    """TL type: help.editUserInfo"""
    CONSTRUCTOR_ID = 0x66b91b70
    SUBCLASS_OF_ID = 0x5c53d7d8

    def __init__(self, user_id: 'TypeInputUser', message: str, entities: List['TypeMessageEntity']):
        self.user_id = user_id
        self.message = message
        self.entities = entities

    def to_dict(self):
        return {
            '_': 'EditUserInfoRequest',
            'user_id': self.user_id,
            'message': self.message,
            'entities': self.entities,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(bytes(self.user_id))
        buf.write(TLObject.serialize_bytes(self.message))
        buf.write(struct.pack('<i', 0x1cb5c415))  # vector id
        buf.write(struct.pack('<i', len(self.entities)))
        for item in self.entities:
            buf.write(bytes(self.item))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_user_id = reader.tgread_object()
        _args['user_id'] = _val_user_id
        _val_message = reader.tgread_string()
        _args['message'] = _val_message
        reader.read_int(signed=False)  # skip vector id
        _count_entities = reader.read_int()
        _list_entities = []
        for _ in range(_count_entities):
            _item_entities = reader.tgread_object()
            _list_entities.append(_item_entities)
        _args['entities'] = _list_entities
        return cls(**_args)


class GetAppConfigRequest(TLRequest):
    """TL type: help.getAppConfig"""
    CONSTRUCTOR_ID = 0x61e3f854
    SUBCLASS_OF_ID = 0x14381c9a

    def __init__(self, hash: int):
        self.hash = hash

    def to_dict(self):
        return {
            '_': 'GetAppConfigRequest',
            'hash': self.hash,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(struct.pack('<i', self.hash))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_hash = reader.read_int()
        _args['hash'] = _val_hash
        return cls(**_args)


class GetAppUpdateRequest(TLRequest):
    """TL type: help.getAppUpdate"""
    CONSTRUCTOR_ID = 0x522d5a7d
    SUBCLASS_OF_ID = 0x5897069e

    def __init__(self, source: str):
        self.source = source

    def to_dict(self):
        return {
            '_': 'GetAppUpdateRequest',
            'source': self.source,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(TLObject.serialize_bytes(self.source))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_source = reader.tgread_string()
        _args['source'] = _val_source
        return cls(**_args)


class GetCdnConfigRequest(TLRequest):
    """TL type: help.getCdnConfig"""
    CONSTRUCTOR_ID = 0x52029342
    SUBCLASS_OF_ID = 0xecda397c

    def __init__(self):
        pass

    def to_dict(self):
        return {
            '_': 'GetCdnConfigRequest',
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        return cls()


class GetConfigRequest(TLRequest):
    """TL type: help.getConfig"""
    CONSTRUCTOR_ID = 0xc4f9186b
    SUBCLASS_OF_ID = 0xd3262a4a

    def __init__(self):
        pass

    def to_dict(self):
        return {
            '_': 'GetConfigRequest',
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        return cls()


class GetCountriesListRequest(TLRequest):
    """TL type: help.getCountriesList"""
    CONSTRUCTOR_ID = 0x735787a8
    SUBCLASS_OF_ID = 0xea31fe88

    def __init__(self, lang_code: str, hash: int):
        self.lang_code = lang_code
        self.hash = hash

    def to_dict(self):
        return {
            '_': 'GetCountriesListRequest',
            'lang_code': self.lang_code,
            'hash': self.hash,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(TLObject.serialize_bytes(self.lang_code))
        buf.write(struct.pack('<i', self.hash))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_lang_code = reader.tgread_string()
        _args['lang_code'] = _val_lang_code
        _val_hash = reader.read_int()
        _args['hash'] = _val_hash
        return cls(**_args)


class GetDeepLinkInfoRequest(TLRequest):
    """TL type: help.getDeepLinkInfo"""
    CONSTRUCTOR_ID = 0x3fedc75f
    SUBCLASS_OF_ID = 0x984aac38

    def __init__(self, path: str):
        self.path = path

    def to_dict(self):
        return {
            '_': 'GetDeepLinkInfoRequest',
            'path': self.path,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(TLObject.serialize_bytes(self.path))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_path = reader.tgread_string()
        _args['path'] = _val_path
        return cls(**_args)


class GetInviteTextRequest(TLRequest):
    """TL type: help.getInviteText"""
    CONSTRUCTOR_ID = 0x4d392343
    SUBCLASS_OF_ID = 0xcf70aa35

    def __init__(self):
        pass

    def to_dict(self):
        return {
            '_': 'GetInviteTextRequest',
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        return cls()


class GetNearestDcRequest(TLRequest):
    """TL type: help.getNearestDc"""
    CONSTRUCTOR_ID = 0x1fb33026
    SUBCLASS_OF_ID = 0x3877045f

    def __init__(self):
        pass

    def to_dict(self):
        return {
            '_': 'GetNearestDcRequest',
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        return cls()


class GetPassportConfigRequest(TLRequest):
    """TL type: help.getPassportConfig"""
    CONSTRUCTOR_ID = 0xc661ad08
    SUBCLASS_OF_ID = 0xc666c0ad

    def __init__(self, hash: int):
        self.hash = hash

    def to_dict(self):
        return {
            '_': 'GetPassportConfigRequest',
            'hash': self.hash,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(struct.pack('<i', self.hash))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_hash = reader.read_int()
        _args['hash'] = _val_hash
        return cls(**_args)


class GetPeerColorsRequest(TLRequest):
    """TL type: help.getPeerColors"""
    CONSTRUCTOR_ID = 0xda80f42f
    SUBCLASS_OF_ID = 0x0e3f6733

    def __init__(self, hash: int):
        self.hash = hash

    def to_dict(self):
        return {
            '_': 'GetPeerColorsRequest',
            'hash': self.hash,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(struct.pack('<i', self.hash))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_hash = reader.read_int()
        _args['hash'] = _val_hash
        return cls(**_args)


class GetPeerProfileColorsRequest(TLRequest):
    """TL type: help.getPeerProfileColors"""
    CONSTRUCTOR_ID = 0xabcfa9fd
    SUBCLASS_OF_ID = 0x0e3f6733

    def __init__(self, hash: int):
        self.hash = hash

    def to_dict(self):
        return {
            '_': 'GetPeerProfileColorsRequest',
            'hash': self.hash,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(struct.pack('<i', self.hash))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_hash = reader.read_int()
        _args['hash'] = _val_hash
        return cls(**_args)


class GetPremiumPromoRequest(TLRequest):
    """TL type: help.getPremiumPromo"""
    CONSTRUCTOR_ID = 0xb81b93d4
    SUBCLASS_OF_ID = 0xc987a338

    def __init__(self):
        pass

    def to_dict(self):
        return {
            '_': 'GetPremiumPromoRequest',
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        return cls()


class GetPromoDataRequest(TLRequest):
    """TL type: help.getPromoData"""
    CONSTRUCTOR_ID = 0xc0977421
    SUBCLASS_OF_ID = 0x9d595542

    def __init__(self):
        pass

    def to_dict(self):
        return {
            '_': 'GetPromoDataRequest',
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        return cls()


class GetRecentMeUrlsRequest(TLRequest):
    """TL type: help.getRecentMeUrls"""
    CONSTRUCTOR_ID = 0x3dc0f114
    SUBCLASS_OF_ID = 0xf269c477

    def __init__(self, referer: str):
        self.referer = referer

    def to_dict(self):
        return {
            '_': 'GetRecentMeUrlsRequest',
            'referer': self.referer,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(TLObject.serialize_bytes(self.referer))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_referer = reader.tgread_string()
        _args['referer'] = _val_referer
        return cls(**_args)


class GetSupportRequest(TLRequest):
    """TL type: help.getSupport"""
    CONSTRUCTOR_ID = 0x9cdf08cd
    SUBCLASS_OF_ID = 0x7159bceb

    def __init__(self):
        pass

    def to_dict(self):
        return {
            '_': 'GetSupportRequest',
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        return cls()


class GetSupportNameRequest(TLRequest):
    """TL type: help.getSupportName"""
    CONSTRUCTOR_ID = 0xd360e72c
    SUBCLASS_OF_ID = 0x7f50b7c2

    def __init__(self):
        pass

    def to_dict(self):
        return {
            '_': 'GetSupportNameRequest',
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        return cls()


class GetTermsOfServiceUpdateRequest(TLRequest):
    """TL type: help.getTermsOfServiceUpdate"""
    CONSTRUCTOR_ID = 0x2ca51fd1
    SUBCLASS_OF_ID = 0x293c2977

    def __init__(self):
        pass

    def to_dict(self):
        return {
            '_': 'GetTermsOfServiceUpdateRequest',
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        return cls()


class GetTimezonesListRequest(TLRequest):
    """TL type: help.getTimezonesList"""
    CONSTRUCTOR_ID = 0x49b30240
    SUBCLASS_OF_ID = 0xca76e475

    def __init__(self, hash: int):
        self.hash = hash

    def to_dict(self):
        return {
            '_': 'GetTimezonesListRequest',
            'hash': self.hash,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(struct.pack('<i', self.hash))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_hash = reader.read_int()
        _args['hash'] = _val_hash
        return cls(**_args)


class GetUserInfoRequest(TLRequest):
    """TL type: help.getUserInfo"""
    CONSTRUCTOR_ID = 0x038a08d3
    SUBCLASS_OF_ID = 0x5c53d7d8

    def __init__(self, user_id: 'TypeInputUser'):
        self.user_id = user_id

    def to_dict(self):
        return {
            '_': 'GetUserInfoRequest',
            'user_id': self.user_id,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(bytes(self.user_id))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_user_id = reader.tgread_object()
        _args['user_id'] = _val_user_id
        return cls(**_args)


class HidePromoDataRequest(TLRequest):
    """TL type: help.hidePromoData"""
    CONSTRUCTOR_ID = 0x1e251c95
    SUBCLASS_OF_ID = 0xf5b399ac

    def __init__(self, peer: 'TypeInputPeer'):
        self.peer = peer

    def to_dict(self):
        return {
            '_': 'HidePromoDataRequest',
            'peer': self.peer,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(bytes(self.peer))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_peer = reader.tgread_object()
        _args['peer'] = _val_peer
        return cls(**_args)


class SaveAppLogRequest(TLRequest):
    """TL type: help.saveAppLog"""
    CONSTRUCTOR_ID = 0x6f02f748
    SUBCLASS_OF_ID = 0xf5b399ac

    def __init__(self, events: List['TypeInputAppEvent']):
        self.events = events

    def to_dict(self):
        return {
            '_': 'SaveAppLogRequest',
            'events': self.events,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(struct.pack('<i', 0x1cb5c415))  # vector id
        buf.write(struct.pack('<i', len(self.events)))
        for item in self.events:
            buf.write(bytes(self.item))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        reader.read_int(signed=False)  # skip vector id
        _count_events = reader.read_int()
        _list_events = []
        for _ in range(_count_events):
            _item_events = reader.tgread_object()
            _list_events.append(_item_events)
        _args['events'] = _list_events
        return cls(**_args)


class SetBotUpdatesStatusRequest(TLRequest):
    """TL type: help.setBotUpdatesStatus"""
    CONSTRUCTOR_ID = 0xec22cfcd
    SUBCLASS_OF_ID = 0xf5b399ac

    def __init__(self, pending_updates_count: int, message: str):
        self.pending_updates_count = pending_updates_count
        self.message = message

    def to_dict(self):
        return {
            '_': 'SetBotUpdatesStatusRequest',
            'pending_updates_count': self.pending_updates_count,
            'message': self.message,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(struct.pack('<i', self.pending_updates_count))
        buf.write(TLObject.serialize_bytes(self.message))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_pending_updates_count = reader.read_int()
        _args['pending_updates_count'] = _val_pending_updates_count
        _val_message = reader.tgread_string()
        _args['message'] = _val_message
        return cls(**_args)

